@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2021 Firejail Authors
+ * Copyright (C) 2014-2022 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -22,6 +22,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <dirent.h>
+#include <limits.h>
 
 typedef struct env_t {
 	struct env_t *next;
@@ -262,7 +263,7 @@ static const char * const env_whitelist[] = {
 	"LANG",
 	"LANGUAGE",
 	"LC_MESSAGES",
-	"PATH",
+	// "PATH",
 	"DISPLAY"	// required by X11
 };
 
@@ -311,6 +312,10 @@ void env_apply_whitelist(void) {
 		errExit("clearenv");
 
 	env_apply_list(env_whitelist, ARRAY_SIZE(env_whitelist));
+
+	// hardcoding PATH
+	if (setenv("PATH", "/usr/local/bin:/usr/bin:/bin:/usr/local/sbin:/usr/sbin:/sbin", 1) < 0)
+		errExit("setenv");
 }
 
 // Filter env variables for a sbox app
