@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2021 Firejail Authors
+ * Copyright (C) 2014-2022 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -20,10 +20,7 @@
 #include "firejail.h"
 #include "../include/gcov_wrapper.h"
 #include <sys/mount.h>
-#include <sys/stat.h>
 #include <sys/statvfs.h>
-#include <sys/wait.h>
-#include <linux/limits.h>
 #include <fnmatch.h>
 #include <glob.h>
 #include <dirent.h>
@@ -35,7 +32,7 @@
 #endif
 
 #define MAX_BUF 4096
-#define EMPTY_STRING ("")
+
 // check noblacklist statements not matched by a proper blacklist in disable-*.inc files
 //#define TEST_NO_BLACKLIST_MATCHING
 
@@ -97,18 +94,9 @@ static void disable_file(OPERATION op, const char *filename) {
 		return;
 	}
 
-	// if the file is not present, do nothing
 	assert(fname);
-	struct stat s;
-	if (stat(fname, &s) < 0) {
-		if (arg_debug)
-			printf("Warning (blacklisting): cannot access %s: %s\n", fname, strerror(errno));
-		free(fname);
-		return;
-	}
-
 	// check for firejail executable
-	// we migth have a file found in ${PATH} pointing to /usr/bin/firejail
+	// we might have a file found in ${PATH} pointing to /usr/bin/firejail
 	// blacklisting it here will end up breaking situations like user clicks on a link in Thunderbird
 	//     and expects Firefox to open in the same sandbox
 	if (strcmp(BINDIR "/firejail", fname) == 0) {
@@ -116,6 +104,24 @@ static void disable_file(OPERATION op, const char *filename) {
 		return;
 	}
 
+	// if the file is not present, do nothing
+	int fd = open(fname, O_PATH|O_CLOEXEC);
+	if (fd < 0) {
+		if (arg_debug)
+			printf("Warning (blacklisting): cannot open %s: %s\n", fname, strerror(errno));
+		free(fname);
+		return;
+	}
+
+	struct stat s;
+	if (fstat(fd, &s) < 0) {
+		if (arg_debug)
+			printf("Warning (blacklisting): cannot stat %s: %s\n", fname, strerror(errno));
+		free(fname);
+		close(fd);
+		return;
+	}
+
 	// modify the file
 	if (op == BLACKLIST_FILE || op == BLACKLIST_NOLOG) {
 		// some distros put all executables under /usr/bin and make /bin a symbolic link
@@ -139,13 +145,6 @@ static void disable_file(OPERATION op, const char *filename) {
 					printf(" - no logging\n");
 			}
 
-			int fd = open(fname, O_PATH|O_CLOEXEC);
-			if (fd < 0) {
-				if (arg_debug)
-					printf("Warning (blacklisting): cannot open %s: %s\n", fname, strerror(errno));
-				free(fname);
-				return;
-			}
 			EUID_ROOT();
 			if (S_ISDIR(s.st_mode)) {
 				if (bind_mount_path_to_fd(RUN_RO_DIR, fd) < 0)
@@ -156,12 +155,24 @@ static void disable_file(OPERATION op, const char *filename) {
 					errExit("disable file");
 			}
 			EUID_USER();
-			close(fd);
 
 			if (op == BLACKLIST_FILE)
 				fs_logger2("blacklist", fname);
 			else
 				fs_logger2("blacklist-nolog", fname);
+
+			// files in /etc will be reprocessed during /etc rebuild
+			if (strncmp(fname, "/etc/", 5) == 0) {
+				ProfileEntry *prf = malloc(sizeof(ProfileEntry));
+				if (!prf)
+					errExit("malloc");
+				memset(prf, 0, sizeof(ProfileEntry));
+				prf->data = strdup(fname);
+				if (!prf->data)
+					errExit("strdup");
+				prf->next = cfg.profile_rebuild_etc;
+				cfg.profile_rebuild_etc = prf;
+			}
 		}
 	}
 	else if (op == MOUNT_READONLY || op == MOUNT_RDWR || op == MOUNT_NOEXEC) {
@@ -170,8 +181,7 @@ static void disable_file(OPERATION op, const char *filename) {
 	else if (op == MOUNT_TMPFS) {
 		if (!S_ISDIR(s.st_mode)) {
 			fwarning("%s is not a directory; cannot mount a tmpfs on top of it.\n", fname);
-			free(fname);
-			return;
+			goto out;
 		}
 
 		uid_t uid = getuid();
@@ -181,19 +191,18 @@ static void disable_file(OPERATION op, const char *filename) {
 			    strncmp(cfg.homedir, fname, strlen(cfg.homedir)) != 0 ||
 			    fname[strlen(cfg.homedir)] != '/') {
 				fwarning("you are not allowed to mount a tmpfs on %s\n", fname);
-				free(fname);
-				return;
+				goto out;
 			}
 		}
 
 		fs_tmpfs(fname, uid);
-		EUID_USER(); // fs_tmpfs returns with EUID 0
-
 		selinux_relabel_path(fname, fname);
 	}
 	else
 		assert(0);
 
+out:
+	close(fd);
 	free(fname);
 }
 
@@ -269,6 +278,8 @@ static void globbing(OPERATION op, const char *pattern, const char *noblacklist[
 
 // blacklist files or directories by mounting empty files on top of them
 void fs_blacklist(void) {
+	EUID_ASSERT();
+
 	ProfileEntry *entry = cfg.profile;
 	if (!entry)
 		return;
@@ -280,7 +291,6 @@ void fs_blacklist(void) {
 	if (noblacklist == NULL)
 		errExit("failed allocating memory for noblacklist entries");
 
-	EUID_USER();
 	while (entry) {
 		OPERATION op = OPERATION_MAX;
 		char *ptr;
@@ -456,8 +466,6 @@ void fs_blacklist(void) {
 	for (i = 0; i < noblacklist_c; i++)
 		free(noblacklist[i]);
 	free(noblacklist);
-
-	EUID_ROOT();
 }
 
 //***********************************************
@@ -466,7 +474,7 @@ void fs_blacklist(void) {
 
 // mount a writable tmpfs on directory; requires a resolved path
 void fs_tmpfs(const char *dir, unsigned check_owner) {
-	EUID_USER();
+	EUID_ASSERT();
 	assert(dir);
 	if (arg_debug)
 		printf("Mounting tmpfs on %s, check owner: %s\n", dir, (check_owner)? "yes": "no");
@@ -489,14 +497,15 @@ void fs_tmpfs(const char *dir, unsigned check_owner) {
 	struct statvfs buf;
 	if (fstatvfs(fd, &buf) == -1)
 		errExit("fstatvfs");
-	unsigned long flags = buf.f_flag & ~(MS_RDONLY|MS_BIND);
+	unsigned long flags = buf.f_flag & ~(MS_RDONLY|MS_BIND|MS_REMOUNT);
 	// mount via the symbolic link in /proc/self/fd
-	EUID_ROOT();
 	char *proc;
 	if (asprintf(&proc, "/proc/self/fd/%d", fd) == -1)
 		errExit("asprintf");
+	EUID_ROOT();
 	if (mount("tmpfs", proc, "tmpfs", flags|MS_NOSUID|MS_NODEV, options) < 0)
 		errExit("mounting tmpfs");
+	EUID_USER();
 	// check the last mount operation
 	MountData *mdata = get_last_mount();
 	if (strcmp(mdata->fstype, "tmpfs") != 0 || strcmp(mdata->dir, dir) != 0)
@@ -622,40 +631,37 @@ out:
 }
 
 // remount recursively; requires a resolved path
-static void fs_remount_rec(const char *dir, OPERATION op) {
+static void fs_remount_rec(const char *path, OPERATION op) {
 	EUID_ASSERT();
-	assert(dir);
+	assert(op < OPERATION_MAX);
+	assert(path);
 
-	struct stat s;
-	if (stat(dir, &s) != 0)
-		return;
-	if (!S_ISDIR(s.st_mode)) {
-		// no need to search in /proc/self/mountinfo for submounts if not a directory
-		fs_remount_simple(dir, op);
+	// no need to search /proc/self/mountinfo for submounts if not a directory
+	int fd = open(path, O_PATH|O_DIRECTORY|O_NOFOLLOW|O_CLOEXEC);
+	if (fd < 0) {
+		fs_remount_simple(path, op);
 		return;
 	}
-	// get mount point of the directory
-	int mountid = get_mount_id(dir);
-	if (mountid == -1)
-		return;
-	if (mountid == -2) {
-		// falling back to a simple remount on old kernels
-		static int mount_warning = 0;
-		if (!mount_warning) {
-			fwarning("read-only, read-write and noexec options are not applied recursively\n");
-			mount_warning = 1;
-		}
-		fs_remount_simple(dir, op);
+
+	// get mount id of the directory
+	int mountid = get_mount_id(fd);
+	close(fd);
+	if (mountid < 0) {
+		// falling back to a simple remount
+		fwarning("%s %s not applied recursively\n", opstr[op], path);
+		fs_remount_simple(path, op);
 		return;
 	}
+
 	// build array with all mount points that need to get remounted
-	char **arr = build_mount_array(mountid, dir);
-	assert(arr);
+	char **arr = build_mount_array(mountid, path);
+	if (!arr)
+		return;
 	// remount
-	char **tmp = arr;
-	while (*tmp) {
-		fs_remount_simple(*tmp, op);
-		free(*tmp++);
+	int i;
+	for (i = 0; arr[i]; i++) {
+		fs_remount_simple(arr[i], op);
+		free(arr[i]);
 	}
 	free(arr);
 }
@@ -819,13 +825,16 @@ void fs_proc_sys_dev_boot(void) {
 // disable firejail configuration in ~/.config/firejail
 void disable_config(void) {
 	EUID_USER();
+#ifndef HAVE_ONLY_SYSCFG_PROFILES
 	char *fname;
 	if (asprintf(&fname, "%s/.config/firejail", cfg.homedir) == -1)
 		errExit("asprintf");
 	disable_file(BLACKLIST_FILE, fname);
 	free(fname);
+#endif
 
 	// disable run time information
+	disable_file(BLACKLIST_FILE, RUN_FIREJAIL_SANDBOX_DIR);
 	disable_file(BLACKLIST_FILE, RUN_FIREJAIL_NETWORK_DIR);
 	disable_file(BLACKLIST_FILE, RUN_FIREJAIL_BANDWIDTH_DIR);
 	disable_file(BLACKLIST_FILE, RUN_FIREJAIL_NAME_DIR);
@@ -890,367 +899,6 @@ void fs_basic_fs(void) {
 }
 
 
-
-#ifdef HAVE_OVERLAYFS
-char *fs_check_overlay_dir(const char *subdirname, int allow_reuse) {
-	assert(subdirname);
-	EUID_ASSERT();
-	struct stat s;
-	char *dirname;
-
-	if (asprintf(&dirname, "%s/.firejail", cfg.homedir) == -1)
-		errExit("asprintf");
-	// check if ~/.firejail already exists
-	if (lstat(dirname, &s) == 0) {
-		if (!S_ISDIR(s.st_mode)) {
-			if (S_ISLNK(s.st_mode))
-				fprintf(stderr, "Error: %s is a symbolic link\n", dirname);
-			else
-				fprintf(stderr, "Error: %s is not a directory\n", dirname);
-			exit(1);
-		}
-		if (s.st_uid != getuid()) {
-			fprintf(stderr, "Error: %s is not owned by the current user\n", dirname);
-			exit(1);
-		}
-	}
-	else {
-		// create ~/.firejail directory
-		create_empty_dir_as_user(dirname, 0700);
-		if (stat(dirname, &s) == -1) {
-			fprintf(stderr, "Error: cannot create directory %s\n", dirname);
-			exit(1);
-		}
-	}
-	free(dirname);
-
-	// check overlay directory
-	if (asprintf(&dirname, "%s/.firejail/%s", cfg.homedir, subdirname) == -1)
-		errExit("asprintf");
-	if (lstat(dirname, &s) == 0) {
-		if (!S_ISDIR(s.st_mode)) {
-			if (S_ISLNK(s.st_mode))
-				fprintf(stderr, "Error: %s is a symbolic link\n", dirname);
-			else
-				fprintf(stderr, "Error: %s is not a directory\n", dirname);
-			exit(1);
-		}
-		if (s.st_uid != 0) {
-			fprintf(stderr, "Error: overlay directory %s is not owned by the root user\n", dirname);
-			exit(1);
-		}
-		if (allow_reuse == 0) {
-			fprintf(stderr, "Error: overlay directory exists, but reuse is not allowed\n");
-			exit(1);
-		}
-	}
-
-	return dirname;
-}
-
-
-
-// mount overlayfs on top of / directory
-// mounting an overlay and chrooting into it:
-//
-// Old Ubuntu kernel
-// # cd ~
-// # mkdir -p overlay/root
-// # mkdir -p overlay/diff
-// # mount -t overlayfs -o lowerdir=/,upperdir=/root/overlay/diff overlayfs /root/overlay/root
-// # chroot /root/overlay/root
-// to shutdown, first exit the chroot and then  unmount the overlay
-// # exit
-// # umount /root/overlay/root
-//
-// Kernels 3.18+
-// # cd ~
-// # mkdir -p overlay/root
-// # mkdir -p overlay/diff
-// # mkdir -p overlay/work
-// # mount -t overlay -o lowerdir=/,upperdir=/root/overlay/diff,workdir=/root/overlay/work overlay /root/overlay/root
-// # cat /etc/mtab | grep overlay
-// /root/overlay /root/overlay/root overlay rw,relatime,lowerdir=/,upperdir=/root/overlay/diff,workdir=/root/overlay/work 0 0
-// # chroot /root/overlay/root
-// to shutdown, first exit the chroot and then  unmount the overlay
-// # exit
-// # umount /root/overlay/root
-
-
-// to do: fix the code below; also, it might work without /dev, but consider keeping /dev/shm; add locking mechanism for overlay-clean
-#include <sys/utsname.h>
-void fs_overlayfs(void) {
-	struct stat s;
-
-	// check kernel version
-	struct utsname u;
-	int rv = uname(&u);
-	if (rv != 0)
-		errExit("uname");
-	int major;
-	int minor;
-	if (2 != sscanf(u.release, "%d.%d", &major, &minor)) {
-		fprintf(stderr, "Error: cannot extract Linux kernel version: %s\n", u.version);
-		exit(1);
-	}
-
-	if (arg_debug)
-		printf("Linux kernel version %d.%d\n", major, minor);
-	int oldkernel = 0;
-	if (major < 3) {
-		fprintf(stderr, "Error: minimum kernel version required 3.x\n");
-		exit(1);
-	}
-	if (major == 3 && minor < 18)
-		oldkernel = 1;
-
-	// mounting an overlayfs on top of / seems to be broken for kernels > 4.19
-	// we disable overlayfs for now, pending fixing
-	if (major >= 4 &&minor >= 19) {
-		fprintf(stderr, "Error: OverlayFS disabled for Linux kernels 4.19 and newer, pending fixing.\n");
-		exit(1);
-	}
-
-	char *oroot = RUN_OVERLAY_ROOT;
-	mkdir_attr(oroot, 0755, 0, 0);
-
-	// set base for working and diff directories
-	char *basedir = RUN_MNT_DIR;
-	int basefd = -1;
-
-	if (arg_overlay_keep) {
-		basedir = cfg.overlay_dir;
-		assert(basedir);
-		// get a file descriptor for ~/.firejail, fails if there is any symlink
-		char *firejail;
-		if (asprintf(&firejail, "%s/.firejail", cfg.homedir) == -1)
-			errExit("asprintf");
-		int fd = safer_openat(-1, firejail, O_PATH|O_DIRECTORY|O_NOFOLLOW|O_CLOEXEC);
-		if (fd == -1)
-			errExit("safer_openat");
-		free(firejail);
-		// create basedir if it doesn't exist
-		// the new directory will be owned by root
-		const char *dirname = gnu_basename(basedir);
-		if (mkdirat(fd, dirname, 0755) == -1 && errno != EEXIST) {
-			perror("mkdir");
-			fprintf(stderr, "Error: cannot create overlay directory %s\n", basedir);
-			exit(1);
-		}
-		// open basedir
-		basefd = openat(fd, dirname, O_PATH|O_DIRECTORY|O_NOFOLLOW|O_CLOEXEC);
-		close(fd);
-	}
-	else {
-		basefd = open(basedir, O_PATH|O_DIRECTORY|O_NOFOLLOW|O_CLOEXEC);
-	}
-	if (basefd == -1) {
-		perror("open");
-		fprintf(stderr, "Error: cannot open overlay directory %s\n", basedir);
-		exit(1);
-	}
-
-	// confirm once more base is owned by root
-	if (fstat(basefd, &s) == -1)
-		errExit("fstat");
-	if (s.st_uid != 0) {
-		fprintf(stderr, "Error: overlay directory %s is not owned by the root user\n", basedir);
-		exit(1);
-	}
-	// confirm permissions of base are 0755
-	if (((S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH) & s.st_mode) != (S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH)) {
-		fprintf(stderr, "Error: invalid permissions on overlay directory %s\n", basedir);
-		exit(1);
-	}
-
-	// create diff and work directories inside base
-	// no need to check arg_overlay_reuse
-	char *odiff;
-	if (asprintf(&odiff, "%s/odiff", basedir) == -1)
-		errExit("asprintf");
-	// the new directory will be owned by root
-	if (mkdirat(basefd, "odiff", 0755) == -1 && errno != EEXIST) {
-		perror("mkdir");
-		fprintf(stderr, "Error: cannot create overlay directory %s\n", odiff);
-		exit(1);
-	}
-	ASSERT_PERMS(odiff, 0, 0, 0755);
-
-	char *owork;
-	if (asprintf(&owork, "%s/owork", basedir) == -1)
-		errExit("asprintf");
-	// the new directory will be owned by root
-	if (mkdirat(basefd, "owork", 0755) == -1 && errno != EEXIST) {
-		perror("mkdir");
-		fprintf(stderr, "Error: cannot create overlay directory %s\n", owork);
-		exit(1);
-	}
-	ASSERT_PERMS(owork, 0, 0, 0755);
-
-	// mount overlayfs
-	if (arg_debug)
-		printf("Mounting OverlayFS\n");
-	char *option;
-	if (oldkernel) { // old Ubuntu/OpenSUSE kernels
-		if (arg_overlay_keep) {
-			fprintf(stderr, "Error: option --overlay= not available for kernels older than 3.18\n");
-			exit(1);
-		}
-		if (asprintf(&option, "lowerdir=/,upperdir=%s", odiff) == -1)
-			errExit("asprintf");
-		if (mount("overlayfs", oroot, "overlayfs", MS_MGC_VAL, option) < 0)
-			errExit("mounting overlayfs");
-	}
-	else { // kernel 3.18 or newer
-		if (asprintf(&option, "lowerdir=/,upperdir=%s,workdir=%s", odiff, owork) == -1)
-			errExit("asprintf");
-		if (mount("overlay", oroot, "overlay", MS_MGC_VAL, option) < 0) {
-			fprintf(stderr, "Debug: running on kernel version %d.%d\n", major, minor);
-			errExit("mounting overlayfs");
-		}
-
-		//***************************
-		// issue #263 start code
-		// My setup has a separate mount point for /home. When the overlay is mounted,
-		// the overlay does not contain the original /home contents.
-		// I added code to create a second overlay for /home if the overlay home dir is empty and this seems to work
-		// @dshmgh, Jan 2016
-		{
-			char *overlayhome;
-			struct stat s;
-			char *hroot;
-			char *hdiff;
-			char *hwork;
-
-			// dons add debug
-			if (arg_debug) printf ("DEBUG: chroot dirs are oroot %s  odiff %s  owork %s\n",oroot,odiff,owork);
-
-			// BEFORE NEXT, WE NEED TO TEST IF /home has any contents or do we need to mount it?
-			// must create var for oroot/cfg.homedir
-			if (asprintf(&overlayhome, "%s%s", oroot, cfg.homedir) == -1)
-				errExit("asprintf");
-			if (arg_debug) printf ("DEBUG: overlayhome var holds ##%s##\n", overlayhome);
-
-			// if no homedir in overlay -- create another overlay for /home
-			if (stat(cfg.homedir, &s) == 0 && stat(overlayhome, &s) == -1) {
-
-				// no need to check arg_overlay_reuse
-				if (asprintf(&hdiff, "%s/hdiff", basedir) == -1)
-					errExit("asprintf");
-				// the new directory will be owned by root
-				if (mkdirat(basefd, "hdiff", 0755) == -1 && errno != EEXIST) {
-					perror("mkdir");
-					fprintf(stderr, "Error: cannot create overlay directory %s\n", hdiff);
-					exit(1);
-				}
-				ASSERT_PERMS(hdiff, 0, 0, 0755);
-
-				// no need to check arg_overlay_reuse
-				if (asprintf(&hwork, "%s/hwork", basedir) == -1)
-					errExit("asprintf");
-				// the new directory will be owned by root
-				if (mkdirat(basefd, "hwork", 0755) == -1 && errno != EEXIST) {
-					perror("mkdir");
-					fprintf(stderr, "Error: cannot create overlay directory %s\n", hwork);
-					exit(1);
-				}
-				ASSERT_PERMS(hwork, 0, 0, 0755);
-
-				// no homedir in overlay so now mount another overlay for /home
-				if (asprintf(&hroot, "%s/home", oroot) == -1)
-					errExit("asprintf");
-				if (asprintf(&option, "lowerdir=/home,upperdir=%s,workdir=%s", hdiff, hwork) == -1)
-					errExit("asprintf");
-				if (mount("overlay", hroot, "overlay", MS_MGC_VAL, option) < 0)
-					errExit("mounting overlayfs for mounted home directory");
-
-				printf("OverlayFS for /home configured in %s directory\n", basedir);
-				free(hroot);
-				free(hdiff);
-				free(hwork);
-
-			} // stat(overlayhome)
-			free(overlayhome);
-		}
-		// issue #263 end code
-		//***************************
-	}
-	fmessage("OverlayFS configured in %s directory\n", basedir);
-	close(basefd);
-
-	// /dev, /run and /tmp are not covered by the overlay
-	// mount-bind dev directory
-	if (arg_debug)
-		printf("Mounting /dev\n");
-	char *dev;
-	if (asprintf(&dev, "%s/dev", oroot) == -1)
-		errExit("asprintf");
-	if (mount("/dev", dev, NULL, MS_BIND|MS_REC, NULL) < 0)
-		errExit("mounting /dev");
-	fs_logger("whitelist /dev");
-
-	// mount-bind run directory
-	if (arg_debug)
-		printf("Mounting /run\n");
-	char *run;
-	if (asprintf(&run, "%s/run", oroot) == -1)
-		errExit("asprintf");
-	if (mount("/run", run, NULL, MS_BIND|MS_REC, NULL) < 0)
-		errExit("mounting /run");
-	fs_logger("whitelist /run");
-
-	// mount-bind tmp directory
-	if (arg_debug)
-		printf("Mounting /tmp\n");
-	char *tmp;
-	if (asprintf(&tmp, "%s/tmp", oroot) == -1)
-		errExit("asprintf");
-	if (mount("/tmp", tmp, NULL, MS_BIND|MS_REC, NULL) < 0)
-		errExit("mounting /tmp");
-	fs_logger("whitelist /tmp");
-
-	// chroot in the new filesystem
-	__gcov_flush();
-
-	if (chroot(oroot) == -1)
-		errExit("chroot");
-
-	// mount a new proc filesystem
-	if (arg_debug)
-		printf("Mounting /proc filesystem representing the PID namespace\n");
-	if (mount("proc", "/proc", "proc", MS_NOSUID | MS_NOEXEC | MS_NODEV | MS_REC, NULL) < 0)
-		errExit("mounting /proc");
-
-	// update /var directory in order to support multiple sandboxes running on the same root directory
-//	if (!arg_private_dev)
-//		fs_dev_shm();
-	fs_var_lock();
-	if (!arg_keep_var_tmp)
-		fs_var_tmp();
-	if (!arg_writable_var_log)
-		fs_var_log();
-	fs_var_lib();
-	fs_var_cache();
-	fs_var_utmp();
-	fs_machineid();
-
-	// don't leak user information
-	restrict_users();
-
-	// when starting as root, firejail config is not disabled;
-	if (getuid() != 0)
-		disable_config();
-
-	// cleanup and exit
-	free(option);
-	free(odiff);
-	free(owork);
-	free(dev);
-	free(run);
-	free(tmp);
-}
-#endif
-
 // this function is called from sandbox.c before blacklist/whitelist functions
 void fs_private_tmp(void) {
 	EUID_ASSERT();
@@ -1274,9 +922,11 @@ void fs_private_tmp(void) {
 
 	// whitelist x11 directory
 	profile_add("whitelist /tmp/.X11-unix");
-	// read-only x11 directory
 	profile_add("read-only /tmp/.X11-unix");
 
+	// whitelist sndio directory
+	profile_add("whitelist /tmp/sndio");
+
 	// whitelist any pulse* file in /tmp directory
 	// some distros use PulseAudio sockets under /tmp instead of the socket in /urn/user
 	DIR *dir;
