@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2021 Firejail Authors
+ * Copyright (C) 2014-2022 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -24,6 +24,7 @@
 #include <sys/types.h>
 #include <time.h>
 #include <unistd.h>
+#include <dirent.h>
 
 // spoof /etc/machine_id
 void fs_machineid(void) {
@@ -103,7 +104,10 @@ static void build_dirs(char *src, char *dst, size_t src_prefix_len, size_t dst_p
 				*q = '\0';
 				*r = '/';
 				r = q;
-				create_empty_dir_as_root(dst, s.st_mode);
+				if (mkdir(dst, 0700) != 0 && errno != EEXIST)
+					errExit("mkdir");
+				if (chmod(dst, s.st_mode) != 0)
+					errExit("chmod");
 			}
 			if (!last) {
 				// If we're not at the final terminating null, restore
@@ -141,7 +145,7 @@ errexit:
 static void duplicate(const char *fname, const char *private_dir, const char *private_run_dir) {
 	assert(fname);
 
-	if (*fname == '~' || *fname == '/' || strncmp(fname, "..", 2) == 0) {
+	if (*fname == '~' || *fname == '/' || strstr(fname, "..")) {
 		fprintf(stderr, "Error: \"%s\" is an invalid filename\n", fname);
 		exit(1);
 	}
@@ -164,7 +168,14 @@ static void duplicate(const char *fname, const char *private_dir, const char *pr
 		errExit("asprintf");
 
 	build_dirs(src, dst, strlen(private_dir), strlen(private_run_dir));
-	sbox_run(SBOX_ROOT | SBOX_SECCOMP, 3, PATH_FCOPY, src, dst);
+
+	// follow links! this will make a copy of the file or directory pointed by the symlink
+	// this will solve problems such as NixOS #4887
+	// don't follow links to dynamic directories such as /proc
+	if (strcmp(src, "/etc/mtab") == 0)
+		sbox_run(SBOX_ROOT | SBOX_SECCOMP, 3, PATH_FCOPY, src, dst);
+	else
+		sbox_run(SBOX_ROOT | SBOX_SECCOMP, 4, PATH_FCOPY, "--follow-link", src, dst);
 
 	free(dst);
 	fs_logger2("clone", src);
@@ -250,3 +261,128 @@ void fs_private_dir_list(const char *private_dir, const char *private_run_dir, c
 	fs_private_dir_mount(private_dir, private_run_dir);
 	fmessage("Private %s installed in %0.2f ms\n", private_dir, timetrace_end());
 }
+
+void fs_rebuild_etc(void) {
+	int have_dhcp = 1;
+	if (cfg.dns1 == NULL && !any_dhcp())
+		have_dhcp = 0;
+
+	if (arg_debug)
+		printf("rebuilding /etc directory\n");
+	if (mkdir(RUN_DNS_ETC, 0755))
+		errExit("mkdir");
+	selinux_relabel_path(RUN_DNS_ETC, "/etc");
+	fs_logger("tmpfs /etc");
+
+	DIR *dir = opendir("/etc");
+	if (!dir)
+		errExit("opendir");
+
+	struct stat s;
+	struct dirent *entry;
+	while ((entry = readdir(dir))) {
+		if (strcmp(entry->d_name, ".") == 0 || strcmp(entry->d_name, "..") == 0)
+			continue;
+
+		// skip files in cfg.profile_rebuild_etc list
+		// these files are already blacklisted
+		{
+			ProfileEntry *prf = cfg.profile_rebuild_etc;
+			int found = 0;
+			while (prf) {
+				if (strcmp(entry->d_name, prf->data + 5) == 0) { // 5 is strlen("/etc/")
+					found = 1;
+					break;
+				}
+				prf = prf->next;
+			}
+			if (found)
+				continue;
+		}
+
+		// for resolv.conf we might have to  create a brand new file later
+		if (have_dhcp &&
+		    (strcmp(entry->d_name, "resolv.conf") == 0 ||
+		     strcmp(entry->d_name, "resolv.conf.dhclient-new") == 0))
+			continue;
+//		printf("linking %s\n", entry->d_name);
+
+		char *src;
+		if (asprintf(&src, "/etc/%s", entry->d_name) == -1)
+			errExit("asprintf");
+		if (stat(src, &s) != 0) {
+			free(src);
+			continue;
+		}
+
+		char *dest;
+		if (asprintf(&dest, "%s/%s", RUN_DNS_ETC, entry->d_name) == -1)
+			errExit("asprintf");
+
+		int symlink_done = 0;
+		if (is_link(src)) {
+			char *rp =realpath(src, NULL);
+			if (rp == NULL) {
+				free(src);
+				free(dest);
+				continue;
+			}
+			if (symlink(rp, dest))
+				errExit("symlink");
+			else
+				symlink_done = 1;
+		}
+		else if (S_ISDIR(s.st_mode))
+			create_empty_dir_as_root(dest, S_IRWXU);
+		else
+			create_empty_file_as_root(dest, S_IRUSR | S_IWUSR);
+
+		// bind-mount src on top of dest
+		if (!symlink_done) {
+			if (mount(src, dest, NULL, MS_BIND|MS_REC, NULL) < 0)
+				errExit("mount bind mirroring /etc");
+		}
+		fs_logger2("clone", src);
+
+		free(src);
+		free(dest);
+	}
+	closedir(dir);
+
+	// mount bind our private etc directory on top of /etc
+	if (arg_debug)
+		printf("Mount-bind %s on top of /etc\n", RUN_DNS_ETC);
+	if (mount(RUN_DNS_ETC, "/etc", NULL, MS_BIND|MS_REC, NULL) < 0)
+		errExit("mount bind mirroring /etc");
+	fs_logger("mount /etc");
+
+	if (have_dhcp == 0)
+		return;
+
+	if (arg_debug)
+		printf("Creating a new /etc/resolv.conf file\n");
+	FILE *fp = fopen("/etc/resolv.conf", "wxe");
+	if (!fp) {
+		fprintf(stderr, "Error: cannot create /etc/resolv.conf file\n");
+		exit(1);
+	}
+
+	if (cfg.dns1) {
+		if (any_dhcp())
+			fwarning("network setup uses DHCP, nameservers will likely be overwritten\n");
+		fprintf(fp, "nameserver %s\n", cfg.dns1);
+	}
+	if (cfg.dns2)
+		fprintf(fp, "nameserver %s\n", cfg.dns2);
+	if (cfg.dns3)
+		fprintf(fp, "nameserver %s\n", cfg.dns3);
+	if (cfg.dns4)
+		fprintf(fp, "nameserver %s\n", cfg.dns4);
+
+	// mode and owner
+	SET_PERMS_STREAM(fp, 0, 0, 0644);
+
+	fclose(fp);
+
+	fs_logger("create /etc/resolv.conf");
+}
