@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2021 Firejail Authors
+ * Copyright (C) 2014-2022 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -19,7 +19,6 @@
 */
 #include "firejail.h"
 #include <sys/mount.h>
-#include <linux/limits.h>
 #include <dirent.h>
 #include <errno.h>
 #include <sys/stat.h>
@@ -34,13 +33,37 @@
 #define O_PATH 010000000
 #endif
 
+static void disable_tab_completion(const char *homedir) {
+	if (arg_tab)
+		return;
+
+	char *fname;
+	if (asprintf(&fname, "%s/.inputrc", homedir) == -1)
+		errExit("asprintf");
+
+	// don't create a new one if we already have it
+	if (access(fname, F_OK)) {
+		FILE *fp = fopen(fname, "w");
+		if (!fp)
+			errExit("fopen");
+		fprintf(fp, "set disable-completion on\n");
+		fclose(fp);
+		if (chmod(fname, 0644))
+			errExit("chmod");
+	}
+	free(fname);
+}
+
+
 static void skel(const char *homedir) {
 	EUID_ASSERT();
+	char *fname;
+
+	disable_tab_completion(homedir);
 
 	// zsh
 	if (!arg_shell_none && (strcmp(cfg.shell,"/usr/bin/zsh") == 0 || strcmp(cfg.shell,"/bin/zsh") == 0)) {
 		// copy skel files
-		char *fname;
 		if (asprintf(&fname, "%s/.zshrc", homedir) == -1)
 			errExit("asprintf");
 		// don't copy it if we already have the file
@@ -65,7 +88,6 @@ static void skel(const char *homedir) {
 	// csh
 	else if (!arg_shell_none && strcmp(cfg.shell,"/bin/csh") == 0) {
 		// copy skel files
-		char *fname;
 		if (asprintf(&fname, "%s/.cshrc", homedir) == -1)
 			errExit("asprintf");
 		// don't copy it if we already have the file
@@ -90,7 +112,6 @@ static void skel(const char *homedir) {
 	// bash etc.
 	else {
 		// copy skel files
-		char *fname;
 		if (asprintf(&fname, "%s/.bashrc", homedir) == -1)
 			errExit("asprintf");
 		// don't copy it if we already have the file
@@ -381,12 +402,14 @@ void fs_private(void) {
 		selinux_relabel_path("/home", "/home");
 		fs_logger("tmpfs /home");
 	}
+	EUID_USER();
 
 	if (u != 0) {
 		if (!arg_allusers && strncmp(homedir, "/home/", 6) == 0) {
 			// create new empty /home/user directory
 			if (arg_debug)
 				printf("Create a new user directory\n");
+			EUID_ROOT();
 			if (mkdir(homedir, S_IRWXU) == -1) {
 				if (mkpath_as_root(homedir) == -1)
 					errExit("mkpath");
@@ -395,7 +418,7 @@ void fs_private(void) {
 			}
 			if (chown(homedir, u, g) < 0)
 				errExit("chown");
-
+			EUID_USER();
 			fs_logger2("mkdir", homedir);
 			fs_logger2("tmpfs", homedir);
 		}
@@ -406,7 +429,6 @@ void fs_private(void) {
 
 		selinux_relabel_path(homedir, homedir);
 	}
-	EUID_USER();
 
 	skel(homedir);
 	if (xflag)
@@ -433,18 +455,33 @@ void fs_check_private_dir(void) {
 }
 
 // check new private working directory (--private-cwd= option) - exit if it fails
+// for testing:
+//    $ firejail --private --private-cwd=. --noprofile ls
+//	issue #4780: exposes full home directory, not the --private one
+//   $ firejail --private-cwd=.. --noprofile ls  -> error: full dir path required
+//   $ firejail --private-cwd=/etc --noprofile ls  -> OK
+//   $ firejail --private-cwd=FULL-SYMLINK-PATH  --noprofile ls  -> error: no symlinks
+//   $ firejail --private --private-cwd="${HOME}" --noprofile ls -al  --> OK
+//   $ firejail --private --private-cwd='${HOME}' --noprofile ls -al  --> OK
+//   $ firejail --private-cwd  --> OK: should go in top of the home dir
+//   profile with "private-cwd ${HOME}
 void fs_check_private_cwd(const char *dir) {
 	EUID_ASSERT();
 	invalid_filename(dir, 0); // no globbing
+	if (strcmp(dir, ".") == 0)
+		goto errout;
 
 	// Expand the working directory
 	cfg.cwd = expand_macros(dir);
 
 	// realpath/is_dir not used because path may not exist outside of jail
-	if (strstr(cfg.cwd, "..")) {
-		fprintf(stderr, "Error: invalid private working directory\n");
-		exit(1);
-	}
+	if (strstr(cfg.cwd, "..") || *cfg.cwd != '/')
+		goto errout;
+
+	return;
+errout:
+	fprintf(stderr, "Error: invalid private working directory\n");
+	exit(1);
 }
 
 //***********************************************************************************
@@ -564,12 +601,13 @@ void fs_private_home_list(void) {
 	int xflag = store_xauthority();
 	int aflag = store_asoundrc();
 
-	// create /run/firejail/mnt/home directory
 	EUID_ROOT();
+	// create /run/firejail/mnt/home directory
 	mkdir_attr(RUN_HOME_DIR, 0755, uid, gid);
 	selinux_relabel_path(RUN_HOME_DIR, homedir);
 
-	fs_logger_print();	// save the current log
+	// save the current log
+	fs_logger_print();
 	EUID_USER();
 
 	// copy the list of files in the new home directory
