@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2021 Firejail Authors
+ * Copyright (C) 2014-2022 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -61,17 +61,31 @@ static int valid_full_path(const char *full_path) {
 	return 0;
 }
 
+// return 1 if symlink to firejail executable
+int is_firejail_link(const char *fname) {
+	EUID_ASSERT();
+
+	if (!is_link(fname))
+		return 0;
+
+	char *rp = realpath(fname, NULL);
+	if (!rp)
+		return 0;
+
+	int rv = 0;
+	const char *base = gnu_basename(rp);
+	if (strcmp(base, "firejail") == 0)
+		rv = 1;
+
+	free(rp);
+	return rv;
+}
+
 char *find_in_path(const char *program) {
 	EUID_ASSERT();
 	if (arg_debug)
 		printf("Searching $PATH for %s\n", program);
 
-	char self[MAXBUF];
-	ssize_t len = readlink("/proc/self/exe", self, MAXBUF - 1);
-	if (len < 0)
-		errExit("readlink");
-	self[len] = '\0';
-
 	const char *path = env_get("PATH");
 	if (!path)
 		return NULL;
@@ -88,18 +102,12 @@ char *find_in_path(const char *program) {
 		if (arg_debug)
 			printf("trying #%s#\n", fname);
 		struct stat s;
-		if (stat(fname, &s) == 0) {
-			// but skip links created by firecfg
-			char *rp = realpath(fname, NULL);
-			if (!rp)
-				errExit("realpath");
-			if (strcmp(self, rp) != 0) {
-				free(rp);
-				free(dup);
-				return fname;
-			}
-			free(rp);
+		if (stat(fname, &s) == 0 &&
+		    !is_firejail_link(fname)) { // skip links created by firecfg
+			free(dup);
+			return fname;
 		}
+
 		free(fname);
 		tok = strtok(NULL, ":");
 	}
@@ -195,6 +203,11 @@ void fslib_mount_libs(const char *full_path, unsigned user) {
 	assert(full_path);
 	// if library/executable does not exist or the user does not have read access to it
 	// print a warning and exit the function.
+	if (access(full_path, F_OK)) {
+		if (arg_debug || arg_debug_private_lib)
+			printf("Cannot find %s, skipping...\n", full_path);
+		return;
+	}
 	if (user && access(full_path, R_OK)) {
 		if (arg_debug || arg_debug_private_lib)
 			printf("Cannot read %s, skipping...\n", full_path);
@@ -263,9 +276,9 @@ static void install_list_entry(const char *lib) {
 	assert(lib);
 
 	// filename check
-	int len = strlen(lib);
-	if (strcspn(lib, "\\&!?\"'<>%^(){}[];,") != (size_t)len ||
-	strstr(lib, "..")) {
+	reject_meta_chars(lib, 1);
+
+	if (strstr(lib, "..")) {
 		fprintf(stderr, "Error: \"%s\" is an invalid library\n", lib);
 		exit(1);
 	}
