@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2021 Firejail Authors
+ * Copyright (C) 2014-2022 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -46,7 +46,8 @@ static void print_file_or_dir(const char *path, const char *fname) {
 	struct stat s;
 	if (stat(name, &s) == -1) {
 		if (lstat(name, &s) == -1) {
-			printf("Error: cannot access %s\n", name);
+			printf("Error: cannot access %s\n", do_replace_cntrl_chars(name, '?'));
+			free(name);
 			return;
 		}
 	}
@@ -151,12 +152,17 @@ static void print_file_or_dir(const char *path, const char *fname) {
 	if (allocated)
 		free(groupname);
 
+	// file size
 	char *sz;
 	if (asprintf(&sz, "%d", (int) s.st_size) == -1)
 		errExit("asprintf");
-	printf("%11.10s %s\n", sz, fname);
-	free(sz);
 
+	// file name
+	char *fname_print = replace_cntrl_chars(fname, '?');
+
+	printf("%11.10s %s\n", sz, fname_print);
+	free(sz);
+	free(fname_print);
 }
 
 static void print_directory(const char *path) {
@@ -192,13 +198,15 @@ void ls(const char *path) {
 		fprintf(stderr, "Error: cannot access %s\n", path);
 		exit(1);
 	}
+
+	// debug doesn't filter control characters currently
 	if (arg_debug)
 		printf("ls %s\n", rp);
 
 	// list directory contents
 	struct stat s;
 	if (stat(rp, &s) == -1) {
-		fprintf(stderr, "Error: cannot access %s\n", rp);
+		fprintf(stderr, "Error: cannot access %s\n", do_replace_cntrl_chars(rp, '?'));
 		exit(1);
 	}
 	if (S_ISDIR(s.st_mode))
@@ -237,13 +245,13 @@ void cat(const char *path) {
 		fprintf(stderr, "Error: %s is not a regular file\n", path);
 		exit(1);
 	}
-	bool tty = isatty(STDOUT_FILENO);
+	int tty = isatty(STDOUT_FILENO);
 
 	int c;
 	while ((c = fgetc(fp)) != EOF) {
 		// file is untrusted
 		// replace control characters when printing to a terminal
-		if (tty && c != '\t' && c != '\n' && iscntrl((unsigned char) c))
+		if (tty && iscntrl((unsigned char) c) && c != '\t' && c != '\n')
 			c = '?';
 		fputc(c, stdout);
 	}
@@ -278,11 +286,7 @@ void sandboxfs(int op, pid_t pid, const char *path1, const char *path2) {
 	EUID_ASSERT();
 	assert(path1);
 
-	// in case the pid is that of a firejail process, use the pid of the first child process
-	pid = switch_to_child(pid);
-
-	// exit if no permission to join the sandbox
-	check_join_permission(pid);
+	ProcessHandle sandbox = pin_sandbox_process(pid);
 
 	// expand paths
 	char *fname1 = expand_path(path1);
@@ -305,7 +309,7 @@ void sandboxfs(int op, pid_t pid, const char *path1, const char *path2) {
 		}
 		// create destination file if necessary
 		EUID_ASSERT();
-		int fd = open(dest_fname, O_WRONLY|O_CREAT|O_CLOEXEC, S_IRUSR | S_IWRITE);
+		int fd = open(dest_fname, O_WRONLY|O_CREAT|O_CLOEXEC, S_IRUSR | S_IWUSR);
 		if (fd == -1) {
 			fprintf(stderr, "Error: cannot open %s for writing\n", dest_fname);
 			exit(1);
@@ -325,23 +329,13 @@ void sandboxfs(int op, pid_t pid, const char *path1, const char *path2) {
 		// redirection
 		if (dup2(fd, STDOUT_FILENO) == -1)
 			errExit("dup2");
-		assert(fd != STDOUT_FILENO);
 		close(fd);
 		op = SANDBOX_FS_CAT;
 	}
 
-	// sandbox root directory
-	char *rootdir;
-	if (asprintf(&rootdir, "/proc/%d/root", pid) == -1)
-		errExit("asprintf");
-
 	if (op == SANDBOX_FS_LS || op == SANDBOX_FS_CAT) {
-		EUID_ROOT();
-		// chroot
-		if (chroot(rootdir) < 0)
-			errExit("chroot");
-		if (chdir("/") < 0)
-			errExit("chdir");
+		// chroot into the sandbox
+		process_rootfs_chroot(sandbox);
 
 		// drop privileges
 		drop_privs(0);
@@ -403,11 +397,8 @@ void sandboxfs(int op, pid_t pid, const char *path1, const char *path2) {
 		if (child < 0)
 			errExit("fork");
 		if (child == 0) {
-			// chroot
-			if (chroot(rootdir) < 0)
-				errExit("chroot");
-			if (chdir("/") < 0)
-				errExit("chdir");
+			// chroot into the sandbox
+			process_rootfs_chroot(sandbox);
 
 			// drop privileges
 			drop_privs(0);
@@ -435,10 +426,6 @@ void sandboxfs(int op, pid_t pid, const char *path1, const char *path2) {
 		EUID_USER();
 	}
 
-	if (fname2)
-		free(fname2);
-	free(fname1);
-	free(rootdir);
-
+	unpin_process(sandbox);
 	exit(0);
 }
