@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2021 Firejail Authors
+ * Copyright (C) 2014-2022 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -32,7 +32,8 @@
 #include <dirent.h>
 #include <pwd.h>
 #include <errno.h>
-//#include <limits.h>
+
+#include <limits.h>
 #include <sys/file.h>
 #include <sys/prctl.h>
 #include <signal.h>
@@ -119,6 +120,7 @@ int arg_whitelist = 0;				// whitelist command
 int arg_nosound = 0;				// disable sound
 int arg_novideo = 0;			//disable video devices in /dev
 int arg_no3d;					// disable 3d hardware acceleration
+int arg_noprinters = 0;				// disable printers
 int arg_quiet = 0;				// no output for scripting
 int arg_join_network = 0;			// join only the network namespace
 int arg_join_filesystem = 0;			// join only the mount namespace
@@ -136,7 +138,7 @@ int arg_allow_debuggers = 0;			// allow debuggers
 int arg_x11_block = 0;				// block X11
 int arg_x11_xorg = 0;				// use X11 security extension
 int arg_allusers = 0;				// all user home directories visible
-int arg_machineid = 0;				// preserve /etc/machine-id
+int arg_machineid = 0;				// spoof /etc/machine-id
 int arg_allow_private_blacklist = 0; 		// blacklist things in private directories
 int arg_disable_mnt = 0;			// disable /mnt and /media
 int arg_noprofile = 0; // use default.profile if none other found/specified
@@ -146,11 +148,14 @@ int arg_nodvd = 0; // --nodvd
 int arg_nou2f = 0; // --nou2f
 int arg_noinput = 0; // --noinput
 int arg_deterministic_exit_code = 0;	// always exit with first child's exit status
+int arg_deterministic_shutdown = 0; 	// shut down the sandbox if first child dies
+int arg_keep_fd_all = 0;		// inherit all file descriptors to sandbox
 DbusPolicy arg_dbus_user = DBUS_POLICY_ALLOW;	// --dbus-user
 DbusPolicy arg_dbus_system = DBUS_POLICY_ALLOW;	// --dbus-system
 const char *arg_dbus_log_file = NULL;
 int arg_dbus_log_user = 0;
 int arg_dbus_log_system = 0;
+int arg_tab = 0;
 int login_shell = 0;
 
 int parent_to_child_fds[2];
@@ -189,13 +194,15 @@ static void my_handler(int s) {
 	logsignal(s);
 
 	if (waitpid(child, NULL, WNOHANG) == 0) {
-		if (has_handler(child, s)) // signals are not delivered if there is no handler yet
+		// child is pid 1 of a pid namespace:
+		// signals are not delivered if there is no handler yet
+		if (has_handler(child, s))
 			kill(child, s);
 		else
 			kill(child, SIGKILL);
 		waitpid(child, NULL, 0);
 	}
-	myexit(s);
+	myexit(128 + s);
 }
 
 static void install_handler(void) {
@@ -349,10 +356,7 @@ static void run_cmd_and_exit(int i, int argc, char **argv) {
 		exit(0);
 	}
 	else if (strcmp(argv[i], "--version") == 0) {
-		printf("firejail version %s\n", VERSION);
-		printf("\n");
-		print_compiletime_support();
-		printf("\n");
+		print_version();
 		exit(0);
 	}
 #ifdef HAVE_OVERLAYFS
@@ -403,6 +407,31 @@ static void run_cmd_and_exit(int i, int argc, char **argv) {
 	}
 #endif
 #ifdef HAVE_NETWORK
+	else if (strcmp(argv[i], "--nettrace") == 0) {
+		if (checkcfg(CFG_NETWORK)) {
+			if (getuid() != 0) {
+				fprintf(stderr, "Error: --nettrace is only available to root user\n");
+				exit(1);
+			}
+			netfilter_trace(0);
+		}
+		else
+			exit_err_feature("networking");
+		exit(0);
+	}
+	else if (strncmp(argv[i], "--nettrace=", 11) == 0) {
+		if (checkcfg(CFG_NETWORK)) {
+			if (getuid() != 0) {
+				fprintf(stderr, "Error: --nettrace is only available to root user\n");
+				exit(1);
+			}
+			pid_t pid = require_pid(argv[i] + 11);
+			netfilter_trace(pid);
+		}
+		else
+			exit_err_feature("networking");
+		exit(0);
+	}
 	else if (strncmp(argv[i], "--bandwidth=", 12) == 0) {
 		if (checkcfg(CFG_NETWORK)) {
 			logargs(argc, argv);
@@ -868,7 +897,7 @@ char *guess_shell(void) {
 	if (shell) {
 		invalid_filename(shell, 0); // no globbing
 		if (access(shell, X_OK) == 0 && !is_dir(shell) && strstr(shell, "..") == NULL &&
-		    strcmp(shell, PATH_FIREJAIL) != 0)
+		    strcmp(gnu_basename(shell), "firejail") != 0)
 			goto found;
 	}
 
@@ -932,12 +961,14 @@ static void run_builder(int argc, char **argv) {
 	if (setresuid(-1, getuid(), getuid()) != 0)
 		errExit("setresuid");
 
+	if (env_get("LD_PRELOAD") != NULL)
+		fprintf(stderr, "run_builder: LD_PRELOAD is: '%s'\n", env_get("LD_PRELOAD"));
 	assert(env_get("LD_PRELOAD") == NULL);
 	assert(getenv("LD_PRELOAD") == NULL);
 	umask(orig_umask);
 
-	// restore some environment variables
-	env_apply_whitelist_sbox();
+	// restore original environment variables
+	env_apply_all();
 
 	argv[0] = LIBDIR "/firejail/fbuilder";
 	execvp(argv[0], argv);
@@ -980,136 +1011,64 @@ int main(int argc, char **argv, char **envp) {
 	int prog_index = -1;			  // index in argv where the program command starts
 	int lockfd_network = -1;
 	int lockfd_directory = -1;
+	int lockfd_sandboxfile = -1;
 	int option_cgroup = 0;
 	int custom_profile = 0;	// custom profile loaded
 	int arg_caps_cmdline = 0; 	// caps requested on command line (used to break out of --chroot)
+	int arg_netlock = 0;
 	char **ptr;
 
-#ifndef HAVE_SUID
-	if (geteuid() != 0) {
-		fprintf(stderr, "Error: Firejail needs to be SUID.\n");
-		fprintf(stderr, "Assuming firejail is installed in /usr/bin, execute the following command as root:\n");
-		fprintf(stderr, "  chmod u+s /usr/bin/firejail\n");
-	}
-#endif
 
 	// sanitize the umask
 	orig_umask = umask(022);
 
-	// check standard streams before printing anything
-	fix_std_streams();
-
 	// drop permissions by default and rise them when required
 	EUID_INIT();
 	EUID_USER();
 
+	// check standard streams before opening any file
+	fix_std_streams();
+
 	// argument count should be larger than 0
 	if (argc == 0 || !argv || strlen(argv[0]) == 0) {
 		fprintf(stderr, "Error: argv is invalid\n");
 		exit(1);
 	} else if (argc >= MAX_ARGS) {
-		fprintf(stderr, "Error: too many arguments\n");
+		fprintf(stderr, "Error: too many arguments: argc (%d) >= MAX_ARGS (%d)\n", argc, MAX_ARGS);
 		exit(1);
 	}
 
+	// sanity check for arguments
+	for (i = 0; i < argc; i++) {
+		if (strlen(argv[i]) >= MAX_ARG_LEN) {
+			fprintf(stderr, "Error: too long arguments: argv[%d] len (%zu) >= MAX_ARG_LEN (%d)\n", i, strlen(argv[i]), MAX_ARG_LEN);
+			exit(1);
+		}
+	}
+
 	// Stash environment variables
 	for (i = 0, ptr = envp; ptr && *ptr && i < MAX_ENVS; i++, ptr++)
 		env_store(*ptr, SETENV);
 
 	// sanity check for environment variables
 	if (i >= MAX_ENVS) {
-		fprintf(stderr, "Error: too many environment variables\n");
+		fprintf(stderr, "Error: too many environment variables: >= MAX_ENVS (%d)\n", MAX_ENVS);
 		exit(1);
 	}
 
-	// sanity check for arguments
-	for (i = 0; i < argc; i++) {
-		if (*argv[i] == 0) {
-			fprintf(stderr, "Error: too short arguments\n");
-			exit(1);
-		}
-		if (strlen(argv[i]) >= MAX_ARG_LEN) {
-			fprintf(stderr, "Error: too long arguments\n");
-			exit(1);
-		}
-	}
-
 	// Reapply a minimal set of environment variables
 	env_apply_whitelist();
 
-	// check if the user is allowed to use firejail
-	init_cfg(argc, argv);
-
-	// get starting timestamp, process --quiet
-	timetrace_start();
+	// process --quiet
 	const char *env_quiet = env_get("FIREJAIL_QUIET");
 	if (check_arg(argc, argv, "--quiet", 1) || (env_quiet && strcmp(env_quiet, "yes") == 0))
 		arg_quiet = 1;
 
-	// cleanup at exit
-	EUID_ROOT();
-	atexit(clear_atexit);
+	// check if the user is allowed to use firejail
+	init_cfg(argc, argv);
 
-	// build /run/firejail directory structure
-	preproc_build_firejail_dir();
-	const char *container_name = env_get("container");
-	if (!container_name || strcmp(container_name, "firejail")) {
-		lockfd_directory = open(RUN_DIRECTORY_LOCK_FILE, O_WRONLY | O_CREAT | O_CLOEXEC, S_IRUSR | S_IWUSR);
-		if (lockfd_directory != -1) {
-			int rv = fchown(lockfd_directory, 0, 0);
-			(void) rv;
-			flock(lockfd_directory, LOCK_EX);
-		}
-		preproc_clean_run();
-		flock(lockfd_directory, LOCK_UN);
-		close(lockfd_directory);
-	}
-	EUID_USER();
-
-	// --ip=dhcp - we need access to /sbin and /usr/sbin directories in order to run ISC DHCP client (dhclient)
-	// these paths are disabled in disable-common.inc
-	if ((i = check_arg(argc, argv, "--ip", 0)) != 0) {
-		if (strncmp(argv[i] + 4, "=dhcp", 5) == 0) {
-			profile_add("noblacklist /sbin");
-			profile_add("noblacklist /usr/sbin");
-		}
-	}
-
-	// for appimages we need to remove "include disable-shell.inc from the profile
-	// a --profile command can show up before --appimage
-	if (check_arg(argc, argv, "--appimage", 1))
-		arg_appimage = 1;
-
-	// process allow-debuggers
-	if (check_arg(argc, argv, "--allow-debuggers", 1)) {
-		// check kernel version
-		struct utsname u;
-		int rv = uname(&u);
-		if (rv != 0)
-			errExit("uname");
-		int major;
-		int minor;
-		if (2 != sscanf(u.release, "%d.%d", &major, &minor)) {
-			fprintf(stderr, "Error: cannot extract Linux kernel version: %s\n", u.version);
-			exit(1);
-		}
-		if (major < 4 || (major == 4 && minor < 8)) {
-			fprintf(stderr, "Error: --allow-debuggers is disabled on Linux kernels prior to 4.8. "
-				"A bug in ptrace call allows a full bypass of the seccomp filter. "
-				"Your current kernel version is %d.%d.\n", major, minor);
-			exit(1);
-		}
-
-		arg_allow_debuggers = 1;
-		char *cmd = strdup("noblacklist ${PATH}/strace");
-		if (!cmd)
-			errExit("strdup");
-		profile_add(cmd);
-	}
-
-	// profile builder
-	if (check_arg(argc, argv, "--build", 0)) // supports both --build and --build=filename
-		run_builder(argc, argv); // this function will not return
+	// get starting timestamp
+	timetrace_start();
 
 	// check argv[0] symlink wrapper if this is not a login shell
 	if (*argv[0] != '-')
@@ -1125,7 +1084,7 @@ int main(int argc, char **argv, char **envp) {
 		EUID_USER();
 		if (rv == 0) {
 			if (check_arg(argc, argv, "--version", 1)) {
-				printf("firejail version %s\n", VERSION);
+				print_version();
 				exit(0);
 			}
 
@@ -1134,15 +1093,53 @@ int main(int argc, char **argv, char **envp) {
 			__builtin_unreachable();
 		}
 	}
-	EUID_ASSERT();
 
+	// profile builder
+	if (check_arg(argc, argv, "--build", 0)) // supports both --build and --build=filename
+		run_builder(argc, argv); // this function will not return
+
+	// intrusion detection system
+#ifdef HAVE_IDS
+	if (check_arg(argc, argv, "--ids-", 0)) // supports both --ids-init and --ids-check
+		run_ids(argc, argv); // this function will not return
+#else
+	if (check_arg(argc, argv, "--ids-", 0)) { // supports both --ids-init and --ids-check
+		fprintf(stderr, "Error: IDS features disabled in your Firejail build.\n"
+			"\tTo enable it, configure your build system using --enable-ids.\n"
+			"\tExample: ./configure --prefix=/usr --enable-ids\n\n");
+		exit(1);
+	}
+#endif
 
-	// check firejail directories
 	EUID_ROOT();
-	delete_run_files(sandbox_pid);
+#ifndef HAVE_SUID
+	if (geteuid() != 0) {
+		fprintf(stderr, "Error: Firejail needs to be SUID.\n");
+		fprintf(stderr, "Assuming firejail is installed in /usr/bin, execute the following command as root:\n");
+		fprintf(stderr, "  chmod u+s /usr/bin/firejail\n");
+	}
+#endif
+
+	// build /run/firejail directory structure
+	preproc_build_firejail_dir();
+	const char *container_name = env_get("container");
+	if (!container_name || strcmp(container_name, "firejail")) {
+		lockfd_directory = open(RUN_DIRECTORY_LOCK_FILE, O_WRONLY | O_CREAT | O_CLOEXEC, S_IRUSR | S_IWUSR);
+		if (lockfd_directory != -1) {
+			int rv = fchown(lockfd_directory, 0, 0);
+			(void) rv;
+			flock(lockfd_directory, LOCK_EX);
+		}
+		preproc_clean_run();
+		flock(lockfd_directory, LOCK_UN);
+		close(lockfd_directory);
+	}
+
+	delete_run_files(getpid());
+	atexit(clear_atexit);
 	EUID_USER();
 
-	//check if the parent is sshd daemon
+	// check if the parent is sshd daemon
 	int parent_sshd = 0;
 	{
 		pid_t ppid = getppid();
@@ -1199,7 +1196,8 @@ int main(int argc, char **argv, char **envp) {
 	}
 	EUID_ASSERT();
 
-	// is this a login shell, or a command passed by sshd, insert command line options from /etc/firejail/login.users
+	// is this a login shell, or a command passed by sshd,
+	// insert command line options from /etc/firejail/login.users
 	if (*argv[0] == '-' || parent_sshd) {
 		if (argc == 1)
 			login_shell = 1;
@@ -1251,10 +1249,55 @@ int main(int argc, char **argv, char **envp) {
 #endif
 	EUID_ASSERT();
 
+	// --ip=dhcp - we need access to /sbin and /usr/sbin directories in order to run ISC DHCP client (dhclient)
+	// these paths are disabled in disable-common.inc
+	if ((i = check_arg(argc, argv, "--ip", 0)) != 0) {
+		if (strncmp(argv[i] + 4, "=dhcp", 5) == 0) {
+			profile_add("noblacklist /sbin");
+			profile_add("noblacklist /usr/sbin");
+		}
+	}
+
+	// process allow-debuggers
+	if (check_arg(argc, argv, "--allow-debuggers", 1)) {
+		// check kernel version
+		struct utsname u;
+		int rv = uname(&u);
+		if (rv != 0)
+			errExit("uname");
+		int major;
+		int minor;
+		if (2 != sscanf(u.release, "%d.%d", &major, &minor)) {
+			fprintf(stderr, "Error: cannot extract Linux kernel version: %s\n", u.version);
+			exit(1);
+		}
+		if (major < 4 || (major == 4 && minor < 8)) {
+			fprintf(stderr, "Error: --allow-debuggers is disabled on Linux kernels prior to 4.8. "
+				"A bug in ptrace call allows a full bypass of the seccomp filter. "
+				"Your current kernel version is %d.%d.\n", major, minor);
+			exit(1);
+		}
+
+		arg_allow_debuggers = 1;
+		char *cmd = strdup("noblacklist ${PATH}/strace");
+		if (!cmd)
+			errExit("strdup");
+		profile_add(cmd);
+	}
+
+	// for appimages we need to remove "include disable-shell.inc from the profile
+	// a --profile command can show up before --appimage
+	if (check_arg(argc, argv, "--appimage", 1))
+		arg_appimage = 1;
+
 	// check for force-nonewprivs in /etc/firejail/firejail.config file
 	if (checkcfg(CFG_FORCE_NONEWPRIVS))
 		arg_nonewprivs = 1;
 
+	// check oom
+	if ((i = check_arg(argc, argv, "--oom=", 0)) != 0)
+		oom_set(argv[i] + 6);
+
 	// parse arguments
 	for (i = 1; i < argc; i++) {
 		run_cmd_and_exit(i, argc, argv); // will exit if the command is recognized
@@ -1526,15 +1569,16 @@ int main(int argc, char **argv, char **envp) {
 		else if (strncmp(argv[i], "--cgroup=", 9) == 0) {
 			if (checkcfg(CFG_CGROUP)) {
 				if (option_cgroup) {
-					fprintf(stderr, "Error: only a cgroup can be defined\n");
+					fprintf(stderr, "Error: only one cgroup can be defined\n");
 					exit(1);
 				}
-
-				option_cgroup = 1;
 				cfg.cgroup = strdup(argv[i] + 9);
 				if (!cfg.cgroup)
 					errExit("strdup");
-				set_cgroup(cfg.cgroup);
+
+				check_cgroup_file(cfg.cgroup);
+				set_cgroup(cfg.cgroup, getpid());
+				option_cgroup = 1;
 			}
 			else
 				exit_err_feature("cgroup");
@@ -1565,6 +1609,7 @@ int main(int argc, char **argv, char **envp) {
 			profile_check_line(line, 0, NULL);	// will exit if something wrong
 			profile_add(line);
 		}
+
 		else if (strncmp(argv[i], "--blacklist=", 12) == 0) {
 			char *line;
 			if (asprintf(&line, "blacklist %s", argv[i] + 12) == -1)
@@ -1581,19 +1626,13 @@ int main(int argc, char **argv, char **envp) {
 			profile_check_line(line, 0, NULL);	// will exit if something wrong
 			profile_add(line);
 		}
-
-#ifdef HAVE_WHITELIST
 		else if (strncmp(argv[i], "--whitelist=", 12) == 0) {
-			if (checkcfg(CFG_WHITELIST)) {
-				char *line;
-				if (asprintf(&line, "whitelist %s", argv[i] + 12) == -1)
-					errExit("asprintf");
+			char *line;
+			if (asprintf(&line, "whitelist %s", argv[i] + 12) == -1)
+				errExit("asprintf");
 
-				profile_check_line(line, 0, NULL);	// will exit if something wrong
-				profile_add(line);
-			}
-			else
-				exit_err_feature("whitelist");
+			profile_check_line(line, 0, NULL);	// will exit if something wrong
+			profile_add(line);
 		}
 		else if (strncmp(argv[i], "--nowhitelist=", 14) == 0) {
 			char *line;
@@ -1603,7 +1642,7 @@ int main(int argc, char **argv, char **envp) {
 			profile_check_line(line, 0, NULL);	// will exit if something wrong
 			profile_add(line);
 		}
-#endif
+
 		else if (strncmp(argv[i], "--mkdir=", 8) == 0) {
 			char *line;
 			if (asprintf(&line, "mkdir %s", argv[i] + 8) == -1)
@@ -1853,6 +1892,14 @@ int main(int argc, char **argv, char **envp) {
 			}
 			profile_add_ignore(argv[i] + 9);
 		}
+		else if (strncmp(argv[i], "--keep-fd=", 10) == 0) {
+			if (strcmp(argv[i] + 10, "all") == 0)
+				arg_keep_fd_all = 1;
+			else {
+				const char *add = argv[i] + 10;
+				profile_list_augment(&cfg.keep_fd, add);
+			}
+		}
 #ifdef HAVE_CHROOT
 		else if (strncmp(argv[i], "--chroot=", 9) == 0) {
 			if (checkcfg(CFG_CHROOT)) {
@@ -2121,6 +2168,11 @@ int main(int argc, char **argv, char **envp) {
 			arg_novideo = 1;
 		else if (strcmp(argv[i], "--no3d") == 0)
 			arg_no3d = 1;
+		else if (strcmp(argv[i], "--noprinters") == 0) {
+			arg_noprinters = 1;
+			profile_add("blacklist /dev/lp*");
+			profile_add("blacklist /run/cups/cups.sock");
+		}
 		else if (strcmp(argv[i], "--notv") == 0)
 			arg_notv = 1;
 		else if (strcmp(argv[i], "--nodvd") == 0)
@@ -2293,6 +2345,21 @@ int main(int argc, char **argv, char **envp) {
 			continue;
 		}
 #ifdef HAVE_NETWORK
+		else if (strcmp(argv[i], "--netlock") == 0) {
+			if (checkcfg(CFG_NETWORK))
+				arg_netlock = 1;
+			else
+				exit_err_feature("networking");
+		}
+		else if (strncmp(argv[i], "--netlock=", 10) == 0) {
+			if (checkcfg(CFG_NETWORK)) {
+				pid_t pid = require_pid(argv[i] + 10);
+				netfilter_netlock(pid);
+			}
+			else
+				exit_err_feature("networking");
+			exit(0);
+		}
 		else if (strncmp(argv[i], "--interface=", 12) == 0) {
 			if (checkcfg(CFG_NETWORK)) {
 				// checks
@@ -2598,7 +2665,7 @@ int main(int argc, char **argv, char **envp) {
 			else if (cfg.dns4 == NULL)
 				cfg.dns4 = dns;
 			else {
-				fwarning("Warning: up to 4 DNS servers can be specified, %s ignored\n", dns);
+				fwarning("up to 4 DNS servers can be specified, %s ignored\n", dns);
 				free(dns);
 			}
 		}
@@ -2619,6 +2686,15 @@ int main(int argc, char **argv, char **envp) {
 			if (checkcfg(CFG_NETWORK)) {
 				arg_netfilter = 1;
 				arg_netfilter_file = argv[i] + 12;
+
+				// expand tilde
+				if (*arg_netfilter_file == '~') {
+					char *tmp;
+					if (asprintf(&tmp, "%s%s", cfg.homedir, arg_netfilter_file + 1) == -1)
+						errExit("asprintf");
+					arg_netfilter_file = tmp;
+				}
+
 				check_netfilter_file(arg_netfilter_file);
 			}
 			else
@@ -2629,6 +2705,15 @@ int main(int argc, char **argv, char **envp) {
 			if (checkcfg(CFG_NETWORK)) {
 				arg_netfilter6 = 1;
 				arg_netfilter6_file = argv[i] + 13;
+
+				// expand tilde
+				if (*arg_netfilter6_file == '~') {
+					char *tmp;
+					if (asprintf(&tmp, "%s%s", cfg.homedir, arg_netfilter6_file + 1) == -1)
+						errExit("asprintf");
+					arg_netfilter6_file = tmp;
+				}
+
 				check_netfilter_file(arg_netfilter6_file);
 			}
 			else
@@ -2649,8 +2734,12 @@ int main(int argc, char **argv, char **envp) {
 		//*************************************
 		else if (strncmp(argv[i], "--timeout=", 10) == 0)
 			cfg.timeout = extract_timeout(argv[i] + 10);
-		else if (strcmp(argv[i], "--appimage") == 0)
-			arg_appimage = 1;
+		else if (strcmp(argv[i], "--appimage") == 0) {
+			// already handled
+		}
+		else if (strncmp(argv[i], "--oom=", 6) == 0) {
+			// already handled
+		}
 		else if (strcmp(argv[i], "--shell=none") == 0) {
 			arg_shell_none = 1;
 			if (cfg.shell) {
@@ -2725,6 +2814,11 @@ int main(int argc, char **argv, char **envp) {
 		else if (strcmp(argv[i], "--deterministic-exit-code") == 0) {
 			arg_deterministic_exit_code = 1;
 		}
+		else if (strcmp(argv[i], "--deterministic-shutdown") == 0) {
+			arg_deterministic_shutdown = 1;
+		}
+		else if (strcmp(argv[i], "--tab") == 0)
+			arg_tab = 1;
 		else {
 			// double dash - positional params to follow
 			if (strcmp(argv[i], "--") == 0) {
@@ -2798,6 +2892,17 @@ int main(int argc, char **argv, char **envp) {
 		}
 	}
 
+	// check writable_etc and DNS/DHCP
+	if (arg_writable_etc) {
+		if (cfg.dns1 != NULL || any_dhcp()) {
+			// we could end up overwriting the real /etc/resolv.conf, so we better exit now!
+			fprintf(stderr, "Error: --dns/--ip=dhcp and --writable-etc are mutually exclusive\n");
+			exit(1);
+		}
+	}
+
+
+
 	// enable seccomp if only seccomp.block-secondary was specified
 	if (arg_seccomp_block_secondary)
 		arg_seccomp = 1;
@@ -3015,6 +3120,9 @@ int main(int argc, char **argv, char **envp) {
 		errExit("clone");
 	EUID_USER();
 
+	// sandbox pidfile
+	lockfd_sandboxfile = set_sandbox_run_file(getpid(), child);
+
 	if (!arg_command && !arg_quiet) {
 		fmessage("Parent pid %u, child pid %u\n", sandbox_pid, child);
 		// print the path of the new log directory
@@ -3048,100 +3156,179 @@ int main(int argc, char **argv, char **envp) {
 	}
 	EUID_ASSERT();
 
- 	// close each end of the unused pipes
- 	close(parent_to_child_fds[0]);
- 	close(child_to_parent_fds[1]);
+	// close each end of the unused pipes
+	close(parent_to_child_fds[0]);
+	close(child_to_parent_fds[1]);
 
 	// notify child that base setup is complete
- 	notify_other(parent_to_child_fds[1]);
+	notify_other(parent_to_child_fds[1]);
 
- 	// wait for child to create new user namespace with CLONE_NEWUSER
- 	wait_for_other(child_to_parent_fds[0]);
- 	close(child_to_parent_fds[0]);
+	// wait for child to create new user namespace with CLONE_NEWUSER
+	wait_for_other(child_to_parent_fds[0]);
+	close(child_to_parent_fds[0]);
 
- 	if (arg_noroot) {
-	 	// update the UID and GID maps in the new child user namespace
+	if (arg_noroot) {
+		// update the UID and GID maps in the new child user namespace
 		// uid
-	 	char *map_path;
-	 	if (asprintf(&map_path, "/proc/%d/uid_map", child) == -1)
-	 		errExit("asprintf");
+		char *map_path;
+		if (asprintf(&map_path, "/proc/%d/uid_map", child) == -1)
+			errExit("asprintf");
 
-	 	char *map;
-	 	uid_t uid = getuid();
-	 	if (asprintf(&map, "%d %d 1", uid, uid) == -1)
-	 		errExit("asprintf");
- 		EUID_ROOT();
-	 	update_map(map, map_path);
-	 	EUID_USER();
-	 	free(map);
-	 	free(map_path);
+		char *map;
+		uid_t uid = getuid();
+		if (asprintf(&map, "%d %d 1", uid, uid) == -1)
+			errExit("asprintf");
+		EUID_ROOT();
+		update_map(map, map_path);
+		EUID_USER();
+		free(map);
+		free(map_path);
 
-	 	// gid file
+		// gid file
 		if (asprintf(&map_path, "/proc/%d/gid_map", child) == -1)
 			errExit("asprintf");
-	 	char gidmap[1024];
-	 	char *ptr = gidmap;
-	 	*ptr = '\0';
+		char gidmap[1024];
+		char *ptr = gidmap;
+		*ptr = '\0';
 
-	 	// add user group
-	 	gid_t gid = getgid();
-	 	sprintf(ptr, "%d %d 1\n", gid, gid);
-	 	ptr += strlen(ptr);
+		// add user group
+		gid_t gid = getgid();
+		sprintf(ptr, "%d %d 1\n", gid, gid);
+		ptr += strlen(ptr);
 
-	 	if (!arg_nogroups) {
-		 	//  add firejail group
-		 	gid_t g = get_group_id("firejail");
-		 	if (g) {
-		 		sprintf(ptr, "%d %d 1\n", g, g);
-		 		ptr += strlen(ptr);
-		 	}
+		gid_t g;
+		if (!arg_nogroups || !check_can_drop_all_groups()) {
+			// add audio group
+			if (!arg_nosound) {
+				g = get_group_id("audio");
+				if (g) {
+					sprintf(ptr, "%d %d 1\n", g, g);
+					ptr += strlen(ptr);
+				}
+			}
 
-		 	//  add tty group
-		 	g = get_group_id("tty");
-		 	if (g) {
-		 		sprintf(ptr, "%d %d 1\n", g, g);
-		 		ptr += strlen(ptr);
-		 	}
+			// add video group
+			if (!arg_novideo) {
+				g = get_group_id("video");
+				if (g) {
+					sprintf(ptr, "%d %d 1\n", g, g);
+					ptr += strlen(ptr);
+				}
+			}
 
-		 	//  add audio group
-		 	g = get_group_id("audio");
-		 	if (g) {
-		 		sprintf(ptr, "%d %d 1\n", g, g);
-		 		ptr += strlen(ptr);
-		 	}
+			// add render/vglusers group
+			if (!arg_no3d) {
+				g = get_group_id("render");
+				if (g) {
+					sprintf(ptr, "%d %d 1\n", g, g);
+					ptr += strlen(ptr);
+				}
+				g = get_group_id("vglusers");
+				if (g) {
+					sprintf(ptr, "%d %d 1\n", g, g);
+					ptr += strlen(ptr);
+				}
+			}
 
-		 	//  add video group
-		 	g = get_group_id("video");
-		 	if (g) {
-		 		sprintf(ptr, "%d %d 1\n", g, g);
-		 		ptr += strlen(ptr);
-		 	}
+			// add lp group
+			if (!arg_noprinters) {
+				g = get_group_id("lp");
+				if (g) {
+					sprintf(ptr, "%d %d 1\n", g, g);
+					ptr += strlen(ptr);
+				}
+			}
 
-		 	//  add games group
-		 	g = get_group_id("games");
-		 	if (g) {
-		 		sprintf(ptr, "%d %d 1\n", g, g);
-		 	}
-		 }
+			// add cdrom/optical groups
+			if (!arg_nodvd) {
+				g = get_group_id("cdrom");
+				if (g) {
+					sprintf(ptr, "%d %d 1\n", g, g);
+					ptr += strlen(ptr);
+				}
+				g = get_group_id("optical");
+				if (g) {
+					sprintf(ptr, "%d %d 1\n", g, g);
+					ptr += strlen(ptr);
+				}
+			}
 
- 		EUID_ROOT();
-	 	update_map(gidmap, map_path);
-	 	EUID_USER();
-	 	free(map_path);
- 	}
+			// add input group
+			if (!arg_noinput) {
+				g = get_group_id("input");
+				if (g) {
+					sprintf(ptr, "%d %d 1\n", g, g);
+					ptr += strlen(ptr);
+				}
+			}
+		}
+
+		if (!arg_nogroups) {
+			// add firejail group
+			g = get_group_id("firejail");
+			if (g) {
+				sprintf(ptr, "%d %d 1\n", g, g);
+				ptr += strlen(ptr);
+			}
+
+			// add tty group
+			g = get_group_id("tty");
+			if (g) {
+				sprintf(ptr, "%d %d 1\n", g, g);
+				ptr += strlen(ptr);
+			}
+
+			// add games group
+			g = get_group_id("games");
+			if (g) {
+				sprintf(ptr, "%d %d 1\n", g, g);
+			}
+		}
+
+		EUID_ROOT();
+		update_map(gidmap, map_path);
+		EUID_USER();
+		free(map_path);
+	}
 	EUID_ASSERT();
 
- 	// notify child that UID/GID mapping is complete
- 	notify_other(parent_to_child_fds[1]);
- 	close(parent_to_child_fds[1]);
+	// notify child that UID/GID mapping is complete
+	notify_other(parent_to_child_fds[1]);
+	close(parent_to_child_fds[1]);
 
- 	EUID_ROOT();
+	EUID_ROOT();
 	if (lockfd_network != -1) {
 		flock(lockfd_network, LOCK_UN);
 		close(lockfd_network);
 	}
 	EUID_USER();
 
+	// lock netfilter firewall
+	if (arg_netlock) {
+		pid_t netlock_child = fork();
+		if (netlock_child < 0)
+			errExit("fork");
+		if (netlock_child == 0) {
+			close_all(NULL, 0);
+			// drop privileges
+			if (setresgid(-1, getgid(), getgid()) != 0)
+				errExit("setresgid");
+			if (setresuid(-1, getuid(), getuid()) != 0)
+				errExit("setresuid");
+
+			char arg[64];
+			snprintf(arg, sizeof(arg), "--netlock=%d", getpid());
+
+			char *cmd[3];
+			cmd[0] = BINDIR "/firejail";
+			cmd[1] = arg;
+			cmd[2] = NULL;
+			execvp(cmd[0], cmd);
+			perror("Cannot start netlock");
+			_exit(1);
+		}
+	}
+
 	int status = 0;
 	//*****************************
 	// following code is signal-safe
@@ -3159,35 +3346,17 @@ int main(int argc, char **argv, char **envp) {
 	// end of signal-safe code
 	//*****************************
 
-#if 0
-// at this point the sandbox was closed and we are on our way out
-// it would make sense to move this before waitpid above to free some memory
-// crash for now as of issue #3662 from dhcp code
-	// free globals
-	if (cfg.profile) {
-		ProfileEntry *prf = cfg.profile;
-		while (prf != NULL) {
-			ProfileEntry *next = prf->next;
-printf("data #%s#\n", prf->data);
-			if (prf->data)
-				free(prf->data);
-printf("link #%s#\n", prf->link);
-			if (prf->link)
-				free(prf->link);
-			free(prf);
-			prf = next;
-		}
-	}
-#endif
-
+	// release lock
+	close(lockfd_sandboxfile);
 
 	if (WIFEXITED(status)){
 		myexit(WEXITSTATUS(status));
 	} else if (WIFSIGNALED(status)) {
-		myexit(WTERMSIG(status));
+		// distinguish fatal signals by adding 128
+		myexit(128 + WTERMSIG(status));
 	} else {
-		myexit(0);
+		myexit(1);
 	}
 
-	return 0;
+	return 1;
 }
