@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2021 Firejail Authors
+ * Copyright (C) 2014-2022 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -38,6 +38,12 @@ void preproc_build_firejail_dir(void) {
 		create_empty_dir_as_root(RUN_FIREJAIL_DIR, 0755);
 	}
 
+	// restricted search permission
+	// only root should be able to lock files in this directory
+	if (stat(RUN_FIREJAIL_SANDBOX_DIR, &s)) {
+		create_empty_dir_as_root(RUN_FIREJAIL_SANDBOX_DIR, 0700);
+	}
+
 	if (stat(RUN_FIREJAIL_NETWORK_DIR, &s)) {
 		create_empty_dir_as_root(RUN_FIREJAIL_NETWORK_DIR, 0755);
 	}
@@ -85,7 +91,10 @@ void preproc_build_firejail_dir(void) {
 	}
 
 	create_empty_file_as_root(RUN_RO_FILE, S_IRUSR);
+	fs_remount(RUN_RO_FILE, MOUNT_READONLY, 0);
+
 	create_empty_dir_as_root(RUN_RO_DIR, S_IRUSR);
+	fs_remount(RUN_RO_DIR, MOUNT_READONLY, 0);
 }
 
 // build /run/firejail/mnt directory
