@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2021 Firejail Authors
+ * Copyright (C) 2014-2022 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -72,6 +72,7 @@ static int profile_find(const char *name, const char *dir, int add_ext) {
 // search and read the profile specified by name from firejail directories
 // return  1 if a profile was found
 int profile_find_firejail(const char *name, int add_ext) {
+#ifndef HAVE_ONLY_SYSCFG_PROFILES
 	// look for a profile in ~/.config/firejail directory
 	char *usercfgdir;
 	if (asprintf(&usercfgdir, "%s/.config/firejail", cfg.homedir) == -1)
@@ -84,6 +85,9 @@ int profile_find_firejail(const char *name, int add_ext) {
 		rv = profile_find(name, SYSCONFDIR, add_ext);
 
 	return rv;
+#else
+	return profile_find(name, SYSCONFDIR, add_ext);
+#endif
 }
 
 //***************************************************
@@ -175,6 +179,10 @@ static int check_allow_drm(void) {
 	return checkcfg(CFG_BROWSER_ALLOW_DRM) != 0;
 }
 
+static int check_allow_tray(void) {
+	return checkcfg(CFG_ALLOW_TRAY) != 0;
+}
+
 Cond conditionals[] = {
 	{"HAS_APPIMAGE", check_appimage},
 	{"HAS_NET", check_netoptions},
@@ -184,6 +192,7 @@ Cond conditionals[] = {
 	{"HAS_X11", check_x11},
 	{"BROWSER_DISABLE_U2F", check_disable_u2f},
 	{"BROWSER_ALLOW_DRM", check_allow_drm},
+	{"ALLOW_TRAY", check_allow_tray},
 	{ NULL, NULL }
 };
 
@@ -285,6 +294,15 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		return 0;
 	}
 
+	if (strncmp(ptr, "keep-fd ", 8) == 0) {
+		if (strcmp(ptr + 8, "all") == 0)
+			arg_keep_fd_all = 1;
+		else {
+			const char *add = ptr + 8;
+			profile_list_augment(&cfg.keep_fd, add);
+		}
+		return 0;
+	}
 	if (strncmp(ptr, "xephyr-screen ", 14) == 0) {
 #ifdef HAVE_X11
 		if (checkcfg(CFG_X11)) {
@@ -375,6 +393,10 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 #endif
 		return 0;
 	}
+	else if (strcmp(ptr, "tab") == 0) {
+		arg_tab = 1;
+		return 0;
+	}
 	else if (strcmp(ptr, "private-cwd") == 0) {
 		cfg.cwd = NULL;
 		arg_private_cwd = 1;
@@ -411,13 +433,7 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		return 0;
 	}
 	else if (strcmp(ptr, "nogroups") == 0) {
-		// nvidia cards require video group; disable nogroups
-		if (access("/dev/nvidiactl", R_OK) == 0 && arg_no3d == 0) {
-			fwarning("Warning: NVIDIA card detected, nogroups command disabled\n");
-			arg_nogroups = 0;
-		}
-		else
-			arg_nogroups = 1;
+		arg_nogroups = 1;
 		return 0;
 	}
 	else if (strcmp(ptr, "nosound") == 0) {
@@ -444,6 +460,12 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		arg_no3d = 1;
 		return 0;
 	}
+	else if (strcmp(ptr, "noprinters") == 0) {
+		arg_noprinters = 1;
+		profile_add("blacklist /dev/lp*");
+		profile_add("blacklist /run/cups/cups.sock");
+		return 0;
+	}
 	else if (strcmp(ptr, "noinput") == 0) {
 		arg_noinput = 1;
 		return 0;
@@ -630,7 +652,7 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 #endif
 		return 0;
 	}
-	else if (strncmp(ptr, "netns  ", 6) == 0) {
+	else if (strncmp(ptr, "netns ", 6) == 0) {
 #ifdef HAVE_NETWORK
 		if (checkcfg(CFG_NETWORK)) {
 			arg_netns = ptr + 6;
@@ -981,10 +1003,10 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 			warning_feature_disabled("seccomp");
 		return 0;
 	}
-	if (strncmp(ptr, "seccomp.32.drop ", 13) == 0) {
+	if (strncmp(ptr, "seccomp.32.drop ", 16) == 0) {
 		if (checkcfg(CFG_SECCOMP)) {
 			arg_seccomp32 = 1;
-			cfg.seccomp_list_drop32 = seccomp_check_list(ptr + 13);
+			cfg.seccomp_list_drop32 = seccomp_check_list(ptr + 16);
 		}
 		else
 			warning_feature_disabled("seccomp");
@@ -1001,10 +1023,10 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 			warning_feature_disabled("seccomp");
 		return 0;
 	}
-	if (strncmp(ptr, "seccomp.32.keep ", 13) == 0) {
+	if (strncmp(ptr, "seccomp.32.keep ", 16) == 0) {
 		if (checkcfg(CFG_SECCOMP)) {
 			arg_seccomp32 = 1;
-			cfg.seccomp_list_keep32 = seccomp_check_list(ptr + 13);
+			cfg.seccomp_list_keep32 = seccomp_check_list(ptr + 16);
 		}
 		else
 			warning_feature_disabled("seccomp");
@@ -1101,7 +1123,7 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		else if (cfg.dns4 == NULL)
 			cfg.dns4 = dns;
 		else {
-			fwarning("Warning: up to 4 DNS servers can be specified, %s ignored\n", dns);
+			fwarning("up to 4 DNS servers can be specified, %s ignored\n", dns);
 			free(dns);
 		}
 		return 0;
@@ -1124,8 +1146,14 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 
 	// cgroup
 	if (strncmp(ptr, "cgroup ", 7) == 0) {
-		if (checkcfg(CFG_CGROUP))
-			set_cgroup(ptr + 7);
+		if (checkcfg(CFG_CGROUP)) {
+			cfg.cgroup = strdup(ptr + 7);
+			if (!cfg.cgroup)
+				errExit("strdup");
+
+			check_cgroup_file(cfg.cgroup);
+			set_cgroup(cfg.cgroup, getpid());
+		}
 		else
 			warning_feature_disabled("cgroup");
 		return 0;
@@ -1581,6 +1609,11 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		return 0;
 	}
 
+	if (strcmp(ptr, "deterministic-shutdown") == 0) {
+		arg_deterministic_shutdown = 1;
+		return 0;
+	}
+
 	// rest of filesystem
 	if (strncmp(ptr, "blacklist ", 10) == 0)
 		ptr += 10;
@@ -1589,22 +1622,8 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 	else if (strncmp(ptr, "noblacklist ", 12) == 0)
 		ptr += 12;
 	else if (strncmp(ptr, "whitelist ", 10) == 0) {
-#ifdef HAVE_WHITELIST
-		if (checkcfg(CFG_WHITELIST)) {
-			arg_whitelist = 1;
-			ptr += 10;
-		}
-		else {
-			static int whitelist_warning_printed = 0;
-			if (!whitelist_warning_printed) {
-				warning_feature_disabled("whitelist");
-				whitelist_warning_printed = 1;
-			}
-			return 0;
-		}
-#else
-		return 0;
-#endif
+		arg_whitelist = 1;
+		ptr += 10;
 	}
 	else if (strncmp(ptr, "nowhitelist ", 12) == 0)
 		ptr += 12;
@@ -1750,6 +1769,18 @@ void profile_read(const char *fname) {
 			continue;
 		}
 
+		if (strncmp(ptr, "whitelist-ro ", 13) == 0) {
+			char *whitelist, *readonly;
+			if (asprintf(&whitelist, "whitelist %s", ptr + 13) == -1)
+				errExit("asprintf");
+			profile_add(whitelist);
+			if (asprintf(&readonly, "read-only %s", ptr + 13) == -1)
+				errExit("asprintf");
+			profile_add(readonly);
+			free(ptr);
+			continue;
+		}
+
 		// process quiet
 		// todo: a quiet in the profile file cannot be disabled by --ignore on command line
 		if (strcmp(ptr, "quiet") == 0) {
@@ -1914,7 +1945,7 @@ char *profile_list_compress(char *list)
 			/* Include non-empty item */
 			if (!*item)
 				in[i] = 0;
-			/* Remove all allready included items */
+			/* Remove all already included items */
 			for (k = 0; k < i; ++k)
 				in[k] = 0;
 			break;
