@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2021 Firejail Authors
+ * Copyright (C) 2014-2022 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -87,9 +87,9 @@ static void sandbox_handler(int sig){
 
 	// broadcast a SIGKILL
 	kill(-1, SIGKILL);
-	flush_stdin();
 
-	exit(sig);
+	flush_stdin();
+	exit(128 + sig);
 }
 
 static void install_handler(void) {
@@ -204,7 +204,7 @@ static void save_umask(void) {
 }
 
 static char *create_join_file(void) {
-	int fd = open(RUN_JOIN_FILE, O_RDWR|O_CREAT|O_EXCL|O_CLOEXEC, S_IRUSR | S_IWRITE | S_IRGRP | S_IROTH);
+	int fd = open(RUN_JOIN_FILE, O_RDWR|O_CREAT|O_EXCL|O_CLOEXEC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
 	if (fd == -1)
 		errExit("open");
 	if (ftruncate(fd, 1) == -1)
@@ -356,6 +356,15 @@ static int monitor_application(pid_t app_pid) {
 		if (arg_debug)
 			printf("Sandbox monitor: waitpid %d retval %d status %d\n", monitored_pid, rv, status);
 
+		if (arg_deterministic_shutdown) {
+			if (arg_debug)
+				printf("Sandbox monitor: monitored process died, shut down the sandbox\n");
+			kill(-1, SIGTERM);
+			usleep(100000);
+			kill(-1, SIGKILL);
+			break;
+		}
+
 		DIR *dir;
 		if (!(dir = opendir("/proc"))) {
 			// sleep 2 seconds and try again
@@ -377,18 +386,6 @@ static int monitor_application(pid_t app_pid) {
 			if ((pid_t) pid == dhclient4_pid || (pid_t) pid == dhclient6_pid)
 				continue;
 
-			// todo: make this generic
-			// Dillo browser leaves a dpid process running, we need to shut it down
-			int found = 0;
-			if (strcmp(cfg.command_name, "dillo") == 0) {
-				char *pidname = pid_proc_comm(pid);
-				if (pidname && strcmp(pidname, "dpid") == 0)
-					found = 1;
-				free(pidname);
-			}
-			if (found)
-				break;
-
 			monitored_pid = pid;
 			break;
 		}
@@ -407,7 +404,6 @@ static void print_time(void) {
 	fmessage("Child process initialized in %.02f ms\n", delta);
 }
 
-
 // check execute permissions for the program
 // this is done typically by the shell
 // we are here because of --shell=none
@@ -464,10 +460,42 @@ static int ok_to_run(const char *program) {
 	return 0;
 }
 
+static void close_file_descriptors(void) {
+	if (arg_keep_fd_all)
+		return;
+
+	if (arg_debug)
+		printf("Closing non-standard file descriptors\n");
+
+	if (!cfg.keep_fd) {
+		close_all(NULL, 0);
+		return;
+	}
+
+	size_t sz = 0;
+	int *keep = str_to_int_array(cfg.keep_fd, &sz);
+	if (!keep) {
+		fprintf(stderr, "Error: invalid keep-fd option\n");
+		exit(1);
+	}
+	close_all(keep, sz);
+	free(keep);
+}
+
+
 void start_application(int no_sandbox, int fd, char *set_sandbox_status) {
-	// set environment
-	if (no_sandbox == 0)
+	if (no_sandbox == 0) {
+		close_file_descriptors();
+
+		// set nice and rlimits
+		if (arg_nice)
+			set_nice(cfg.nice);
+		set_rlimits();
+
 		env_defaults();
+	}
+
+	// set environment
 	env_apply_all();
 
 	// restore original umask
@@ -798,7 +826,7 @@ int sandbox(void* sandbox_arg) {
 
 	// trace pre-install
 	if (need_preload)
-		fs_trace_preload();
+		fs_trace_touch_or_store_preload();
 
 	// store hosts file
 	if (cfg.hosts_file)
@@ -814,8 +842,11 @@ int sandbox(void* sandbox_arg) {
 		//****************************
 		// trace pre-install, this time inside chroot
 		//****************************
-		if (need_preload)
-			fs_trace_preload();
+		if (need_preload) {
+			int rv = unlink(RUN_LDPRELOAD_FILE);
+			(void) rv;
+			fs_trace_touch_or_store_preload();
+		}
 	}
 	else
 #endif
@@ -887,16 +918,16 @@ int sandbox(void* sandbox_arg) {
 		else if (arg_overlay)
 			fwarning("private-bin feature is disabled in overlay\n");
 		else {
+			EUID_USER();
 			// for --x11=xorg we need to add xauth command
 			if (arg_x11_xorg) {
-				EUID_USER();
 				char *tmp;
 				if (asprintf(&tmp, "%s,xauth", cfg.bin_private_keep) == -1)
 					errExit("asprintf");
 				cfg.bin_private_keep = tmp;
-				EUID_ROOT();
 			}
 			fs_private_bin_list();
+			EUID_ROOT();
 		}
 	}
 
@@ -992,7 +1023,7 @@ int sandbox(void* sandbox_arg) {
 
 			// create /etc/ld.so.preload file again
 			if (need_preload)
-				fs_trace_preload();
+				fs_trace_touch_preload();
 
 			// openSUSE configuration is split between /etc and /usr/etc
 			// process private-etc a second time
@@ -1004,10 +1035,12 @@ int sandbox(void* sandbox_arg) {
 	// apply the profile file
 	//****************************
 	// apply all whitelist commands ...
+	EUID_USER();
 	fs_whitelist();
 
 	// ... followed by blacklist commands
 	fs_blacklist(); // mkdir and mkfile are processed all over again
+	EUID_ROOT();
 
 	//****************************
 	// nosound/no3d/notv/novideo and fix for pulseaudio 7.0
@@ -1016,6 +1049,9 @@ int sandbox(void* sandbox_arg) {
 		// disable pulseaudio
 		pulseaudio_disable();
 
+		// disable pipewire
+		pipewire_disable();
+
 		// disable /dev/snd
 		fs_dev_disable_sound();
 	}
@@ -1041,9 +1077,10 @@ int sandbox(void* sandbox_arg) {
 		fs_dev_disable_input();
 
 	//****************************
-	// set dns
+	// rebuild etc directory, set dns
 	//****************************
-	fs_resolvconf();
+	if (!arg_writable_etc)
+		fs_rebuild_etc();
 
 	//****************************
 	// start dhcp client
@@ -1056,6 +1093,11 @@ int sandbox(void* sandbox_arg) {
 	EUID_USER();
 	int cwd = 0;
 	if (cfg.cwd) {
+		if (is_link(cfg.cwd)) {
+			fprintf(stderr, "Error: unable to enter private working directory: %s\n", cfg.cwd);
+			exit(1);
+		}
+
 		if (chdir(cfg.cwd) == 0)
 			cwd = 1;
 		else if (arg_private_cwd) {
@@ -1223,7 +1265,7 @@ int sandbox(void* sandbox_arg) {
 	//****************************************
 	// drop privileges
 	//****************************************
-	drop_privs(arg_nogroups);
+	drop_privs(0);
 
 	// kill the sandbox in case the parent died
 	prctl(PR_SET_PDEATHSIG, SIGKILL, 0, 0, 0);
@@ -1243,28 +1285,25 @@ int sandbox(void* sandbox_arg) {
 
 	if (app_pid == 0) {
 #ifdef HAVE_APPARMOR
-		// add apparmor confinement after the execve
 		set_apparmor();
 #endif
-
-		// set nice and rlimits
-		if (arg_nice)
-			set_nice(cfg.nice);
-		set_rlimits();
-
 		start_application(0, -1, set_sandbox_status);
 	}
 
 	munmap(set_sandbox_status, 1);
 
 	int status = monitor_application(app_pid);	// monitor application
-	flush_stdin();
 
 	if (WIFEXITED(status)) {
 		// if we had a proper exit, return that exit status
-		return WEXITSTATUS(status);
+		status = WEXITSTATUS(status);
+	} else if (WIFSIGNALED(status)) {
+		// distinguish fatal signals by adding 128
+		status = 128 + WTERMSIG(status);
 	} else {
-		// something else went wrong!
-		return -1;
+		status = -1;
 	}
+
+	flush_stdin();
+	return status;
 }
