@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2021 Firejail Authors
+ * Copyright (C) 2014-2022 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -21,6 +21,7 @@
 #include "firejail.h"
 #include "../include/seccomp.h"
 #include <sys/mman.h>
+#include <sys/wait.h>
 
 typedef struct filter_list {
 	struct filter_list *next;
@@ -70,9 +71,11 @@ int seccomp_install_filters(void) {
 			assert(fl->fname);
 			if (arg_debug)
 				printf("Installing %s seccomp filter\n", fl->fname);
-
+#ifdef SECCOMP_FILTER_FLAG_LOG
+			if (syscall(SYS_seccomp, SECCOMP_SET_MODE_FILTER, SECCOMP_FILTER_FLAG_LOG, &fl->prog)) {
+#else
 			if (prctl(PR_SET_SECCOMP, SECCOMP_MODE_FILTER, &fl->prog)) {
-
+#endif
 				if (!err_printed)
 					fwarning("seccomp disabled, it requires a Linux kernel version 3.5 or newer.\n");
 				err_printed = 1;
@@ -423,26 +426,20 @@ int seccomp_filter_mdwx(bool native) {
 void seccomp_print_filter(pid_t pid) {
 	EUID_ASSERT();
 
-	// in case the pid is that of a firejail process, use the pid of the first child process
-	pid = switch_to_child(pid);
+	ProcessHandle sandbox = pin_sandbox_process(pid);
 
-	// exit if no permission to join the sandbox
-	check_join_permission(pid);
+	// chroot in the sandbox
+	process_rootfs_chroot(sandbox);
+	unpin_process(sandbox);
+
+	drop_privs(0);
 
 	// find the seccomp list file
-	EUID_ROOT();
-	char *fname;
-	if (asprintf(&fname, "/proc/%d/root%s", pid, RUN_SECCOMP_LIST) == -1)
-		errExit("asprintf");
-
-	struct stat s;
-	if (stat(fname, &s) == -1)
-		goto errexit;
-
-	FILE *fp = fopen(fname, "re");
-	if (!fp)
-		goto errexit;
-	free(fname);
+	FILE *fp = fopen(RUN_SECCOMP_LIST, "re");
+	if (!fp) {
+		printf("Cannot access seccomp filter.\n");
+		exit(1);
+	}
 
 	char buf[MAXBUF];
 	while (fgets(buf, MAXBUF, fp)) {
@@ -451,21 +448,21 @@ void seccomp_print_filter(pid_t pid) {
 		if (ptr)
 			*ptr = '\0';
 
-		if (asprintf(&fname, "/proc/%d/root%s", pid, buf) == -1)
-			errExit("asprintf");
-		printf("FILE: %s\n", fname); fflush(0);
+		printf("FILE: %s\n", buf); fflush(0);
 
-		// read and print the filter - run this as root, the user doesn't have access
-		sbox_run(SBOX_ROOT | SBOX_SECCOMP, 2, PATH_FSEC_PRINT, fname);
-		fflush(0);
+		// read and print the filter
+		pid_t child = fork();
+		if (child < 0)
+			errExit("fork");
+		if (child == 0) {
+			execl(PATH_FSEC_PRINT, PATH_FSEC_PRINT, buf, NULL);
+			errExit("execl");
+		}
+		waitpid(child, NULL, 0);
 
 		printf("\n"); fflush(0);
-		free(fname);
 	}
 	fclose(fp);
-	exit(0);
 
-errexit:
-	printf("Cannot access seccomp filter.\n");
-	exit(1);
+	exit(0);
 }
