@@ -1,5 +1,5 @@
  /*
- * Copyright (C) 2014-2021 Firejail Authors
+ * Copyright (C) 2014-2022 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -17,10 +17,8 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <assert.h>
+
+#include "../include/common.h"
 
 #define MAXBUF 2048
 // stats
@@ -67,19 +65,19 @@ static int arg_dbus_system_none = 0;
 static int arg_dbus_user_none = 0;
 static int arg_whitelisthome = 0;
 static int arg_noroot = 0;
-
+static int arg_print_blacklist = 0;
+static int arg_print_whitelist = 0;
 
 static char *profile = NULL;
 
-
 static void usage(void) {
-	printf("proftool - print profile statistics\n");
-	printf("Usage: proftool [options] file[s]\n");
+	printf("profstats - print profile statistics\n");
+	printf("Usage: profstats [options] file[s]\n");
 	printf("Options:\n");
 	printf("   --apparmor - print profiles without apparmor\n");
 	printf("   --caps - print profiles without caps\n");
-	printf("   --dbus-system-none - profiles without  \"dbus-system none\"\n");
-	printf("   --dbus-user-none - profiles without  \"dbus-user none\"\n");
+	printf("   --dbus-system-none - print profiles without \"dbus-system none\"\n");
+	printf("   --dbus-user-none - print profiles without \"dbus-user none\"\n");
 	printf("   --ssh - print profiles without \"include disable-common.inc\"\n");
 	printf("   --noexec - print profiles without \"include disable-exec.inc\"\n");
 	printf("   --noroot - print profiles without \"noroot\"\n");
@@ -87,8 +85,10 @@ static void usage(void) {
 	printf("   --private-dev - print profiles without private-dev\n");
 	printf("   --private-etc - print profiles without private-etc\n");
 	printf("   --private-tmp - print profiles without private-tmp\n");
+	printf("   --print-blacklist - print all --blacklist for a profile\n");
+	printf("   --print-whitelist - print all --private and --whitelist for a profile\n");
 	printf("   --seccomp - print profiles without seccomp\n");
-	printf("   --memory-deny-write-execute - profile without \"memory-deny-write-execute\"\n");
+	printf("   --memory-deny-write-execute - print profiles without \"memory-deny-write-execute\"\n");
 	printf("   --whitelist-home - print profiles whitelisting home directory\n");
 	printf("   --whitelist-var - print profiles without \"include whitelist-var-common.inc\"\n");
 	printf("   --whitelist-runuser - print profiles without \"include whitelist-runuser-common.inc\" or \"blacklist ${RUNUSER}\"\n");
@@ -97,8 +97,9 @@ static void usage(void) {
 	printf("\n");
 }
 
-void process_file(const char *fname) {
+static void process_file(char *fname) {
 	assert(fname);
+	char *tmpfname = NULL;
 
 	if (arg_debug)
 		printf("processing #%s#\n", fname);
@@ -107,9 +108,19 @@ void process_file(const char *fname) {
 
 	FILE *fp = fopen(fname, "r");
 	if (!fp) {
-		fprintf(stderr, "Warning: cannot open %s, while processing %s\n", fname, profile);
-		level--;
-		return;
+		// the file was not found in the current directory
+		// look for it in /etc/firejail directory
+		if (asprintf(&tmpfname, "%s/%s", SYSCONFDIR, fname) == -1)
+			errExit("asprintf");
+
+		fp = fopen(tmpfname, "r");
+		if (!fp) {
+			fprintf(stderr, "Warning: cannot open %s or %s, while processing %s\n", fname, tmpfname, profile);
+			free(tmpfname);
+			level--;
+			return;
+		}
+		fname = tmpfname;
 	}
 
 	int have_include_local = 0;
@@ -125,6 +136,18 @@ void process_file(const char *fname) {
 		if (*ptr == '\n' || *ptr == '#')
 			continue;
 
+		if (arg_print_blacklist) {
+			if (strncmp(ptr, "blacklist", 9) == 0 ||
+			    strncmp(ptr, "noblacklist", 11) == 0)
+				printf("%s: %s\n", fname, ptr);
+		}
+		else if (arg_print_whitelist) {
+			if (strncmp(ptr, "whitelist", 9) == 0 ||
+			    strncmp(ptr, "nowhitelist", 11) == 0 ||
+			    strncmp(ptr, "private", 7) == 0)
+				printf("%s: %s\n", fname, ptr);
+		}
+
 		if (strncmp(ptr, "seccomp", 7) == 0)
 			cnt_seccomp++;
 		else if (strncmp(ptr, "caps", 4) == 0)
@@ -190,6 +213,8 @@ void process_file(const char *fname) {
 	if (!have_include_local)
 		printf("No include .local found in %s\n", fname);
 	level--;
+	if (tmpfname)
+		free(tmpfname);
 }
 
 int main(int argc, char **argv) {
@@ -227,6 +252,10 @@ int main(int argc, char **argv) {
 			arg_privatetmp = 1;
 		else if (strcmp(argv[i], "--private-etc") == 0)
 			arg_privateetc = 1;
+		else if (strcmp(argv[i], "--print-blacklist") == 0)
+			arg_print_blacklist = 1;
+		else if (strcmp(argv[i], "--print-whitelist") == 0)
+			arg_print_whitelist = 1;
 		else if (strcmp(argv[i], "--whitelist-home") == 0)
 			arg_whitelisthome = 1;
 		else if (strcmp(argv[i], "--whitelist-var") == 0)
@@ -347,6 +376,9 @@ int main(int argc, char **argv) {
 		assert(level == 0);
 	}
 
+	if (arg_print_blacklist || arg_print_whitelist)
+		return 0;
+
 	printf("\n");
 	printf("Stats:\n");
 	printf("    profiles\t\t\t%d\n", cnt_profiles);
